@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2008, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -23,7 +23,6 @@
 
 #include "vt_defs.h"
 
-
 typedef struct {
         uint32_t vampir_file_id;
         uint32_t vampir_file_group_id;
@@ -37,7 +36,8 @@ EXTERN vampir_file_t *fd_to_vampirid;
 
 EXTERN vampir_file_t *get_vampir_file(int fd);
 EXTERN int get_max_open_files(void);
-EXTERN int get_total_open_files(int max_open_files);
 EXTERN void vt_iofile_open(const char* fname, int fd);
+EXTERN uint32_t vt_iofile_id(const char* fname);
+EXTERN void vt_iofile_dupfd(int oldfd, int newfd);
 
 #endif /*VT_IOWRAP_HELPER_H_*/
