@@ -9,11 +9,12 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2009 Mellanox Technologies. All rights reserved.
  * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
+ * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,7 +28,7 @@
 
 #include "opal/mca/installdirs/installdirs.h"
 #include "orte/util/show_help.h"
-#include "opal/util/argv.h"
+#include "opal/util/output.h"
 #include "opal/mca/base/mca_base_param.h"
 #include "btl_openib.h"
 #include "btl_openib_mca.h"
@@ -150,15 +151,18 @@ int btl_openib_register_mca_params(void)
 
     CHECK(reg_int("warn_no_device_params_found",
                   "warn_no_hca_params_found",
-                  "Warn when no device-specific parameters are found in the INI file specified by the btl_openib_device_param_files MCA parameter (0 = do not warn; any other value = warn)",
+                  "Warn when no device-specific parameters are found in the INI file specified by the btl_openib_device_param_files MCA parameter "
+                  "(0 = do not warn; any other value = warn)",
                   1, &ival, 0));
     mca_btl_openib_component.warn_no_device_params_found = (0 != ival);
     CHECK(reg_int("warn_default_gid_prefix", NULL,
-                  "Warn when there is more than one active ports and at least one of them connected to the network with only default GID prefix configured (0 = do not warn; any other value = warn)",
+                  "Warn when there is more than one active ports and at least one of them connected to the network with only default GID prefix configured "
+                  "(0 = do not warn; any other value = warn)",
                   1, &ival, 0));
     mca_btl_openib_component.warn_default_gid_prefix = (0 != ival);
     CHECK(reg_int("warn_nonexistent_if", NULL,
-                  "Warn if non-existent devices and/or ports are specified in the btl_openib_if_[in|ex]clude MCA parameters (0 = do not warn; any other value = warn)",
+                  "Warn if non-existent devices and/or ports are specified in the btl_openib_if_[in|ex]clude MCA parameters "
+                  "(0 = do not warn; any other value = warn)",
                   1, &ival, 0));
     mca_btl_openib_component.warn_nonexistent_if = (0 != ival);
 
@@ -194,7 +198,8 @@ int btl_openib_register_mca_params(void)
     free(str);
 
     CHECK(reg_string("device_type", NULL,
-                     "Specify to only use IB or iWARP network adapters (infiniband = only use InfiniBand HCAs; iwarp = only use iWARP NICs; all = use any available adapters)",
+                     "Specify to only use IB or iWARP network adapters "
+                     "(infiniband = only use InfiniBand HCAs; iwarp = only use iWARP NICs; all = use any available adapters)",
                      "all", &str, 0));
     if (0 == strcasecmp(str, "ib") ||
         0 == strcasecmp(str, "infiniband")) {
@@ -218,7 +223,8 @@ int btl_openib_register_mca_params(void)
                   -1, &mca_btl_openib_component.ib_max_btls,
                   REGINT_NEG_ONE_OK | REGINT_GE_ONE));
     CHECK(reg_int("free_list_num", NULL,
-                  "Intial size of free lists (must be >= 1)",
+                  "Initial size of free lists "
+                  "(must be >= 1)",
                   8, &mca_btl_openib_component.ib_free_list_num,
                   REGINT_GE_ONE));
     CHECK(reg_int("free_list_max", NULL,
@@ -227,11 +233,12 @@ int btl_openib_register_mca_params(void)
                   -1, &mca_btl_openib_component.ib_free_list_max,
                   REGINT_NEG_ONE_OK | REGINT_GE_ONE));
     CHECK(reg_int("free_list_inc", NULL,
-                  "Increment size of free lists (must be >= 1)",
+                  "Increment size of free lists "
+                  "(must be >= 1)",
                   32, &mca_btl_openib_component.ib_free_list_inc,
                   REGINT_GE_ONE));
     CHECK(reg_string("mpool", NULL,
-                     "Name of the memory pool to be used (it is unlikely that you will ever want to change this",
+                     "Name of the memory pool to be used (it is unlikely that you will ever want to change this)",
                      "rdma", &mca_btl_openib_component.ib_mpool_name,
                      0));
     CHECK(reg_int("reg_mru_len", NULL,
@@ -251,8 +258,7 @@ int btl_openib_register_mca_params(void)
 
     CHECK(reg_int("max_inline_data", "ib_max_inline_data",
                   "Maximum size of inline data segment "
-                  "(-1 = run-time probe to discover max value, "
-                  "otherwise must be >= 0). "
+                  "(-1 = run-time probe to discover max value, otherwise must be >= 0). "
                   "If not explicitly set, use max_inline_data from "
                   "the INI file containing device-specific parameters",
                   -1, &ival, REGINT_NEG_ONE_OK | REGINT_GE_ZERO));
@@ -260,7 +266,7 @@ int btl_openib_register_mca_params(void)
 
     CHECK(reg_string("pkey", "ib_pkey_val", 
                      "OpenFabrics partition key (pkey) value. "
-                     "Unsigned integer decimal or hex values are allowed (e.g., \"3\" or \"0x3f\") and will be masked against the maximum allowable IB paritition key value (0x7fff)",
+                     "Unsigned integer decimal or hex values are allowed (e.g., \"3\" or \"0x3f\") and will be masked against the maximum allowable IB partition key value (0x7fff)",
                      "0", &pkey, 0));
     mca_btl_openib_component.ib_pkey_val = 
         ompi_btl_openib_ini_intify(pkey) & MCA_BTL_IB_PKEY_MASK;
@@ -316,7 +322,8 @@ int btl_openib_register_mca_params(void)
     }
     mca_btl_openib_component.ib_min_rnr_timer = (uint32_t) ival;
 
-    CHECK(reg_int("ib_timeout", NULL, "InfiniBand transmit timeout, plugged into formula: 4.096 microseconds * (2^btl_openib_ib_timeout)"
+    CHECK(reg_int("ib_timeout", NULL,
+                  "InfiniBand transmit timeout, plugged into formula: 4.096 microseconds * (2^btl_openib_ib_timeout) "
                   "(must be >= 0 and <= 31)",
                   20, &ival, 0));
     if (ival > 31) {
@@ -332,7 +339,8 @@ int btl_openib_register_mca_params(void)
     }
     mca_btl_openib_component.ib_timeout = (uint32_t) ival;
 
-    CHECK(reg_int("ib_retry_count", NULL, "InfiniBand transmit retry count "
+    CHECK(reg_int("ib_retry_count", NULL,
+                  "InfiniBand transmit retry count "
                   "(must be >= 0 and <= 7)",
                   7, &ival, 0));
     if (ival > 7) {
@@ -348,7 +356,8 @@ int btl_openib_register_mca_params(void)
     }
     mca_btl_openib_component.ib_retry_count = (uint32_t) ival;
 
-    CHECK(reg_int("ib_rnr_retry", NULL, "InfiniBand \"receiver not ready\" "
+    CHECK(reg_int("ib_rnr_retry", NULL,
+                  "InfiniBand \"receiver not ready\" "
                   "retry count; applies *only* to SRQ/XRC queues.  PP queues "
                   "use RNR retry values of 0 because Open MPI performs "
                   "software flow control to guarantee that RNRs never occur "
@@ -389,6 +398,15 @@ int btl_openib_register_mca_params(void)
     }
     mca_btl_openib_component.ib_service_level = (uint32_t) ival;
 
+#if (ENABLE_DYNAMIC_SL)
+    CHECK(reg_int("ib_path_record_service_level", NULL,
+                  "Enable getting InfiniBand service level from PathRecord "
+                  "(must be >= 0, 0 = disabled, positive = try to get the "
+                  "service level from PathRecord)",
+                  0, &ival, REGINT_GE_ZERO));
+    mca_btl_openib_component.ib_path_record_service_level = (uint32_t) ival;
+#endif
+
     CHECK(reg_int("use_eager_rdma", NULL, "Use RDMA for eager messages "
                   "(-1 = use device default, 0 = do not use eager RDMA, "
                   "1 = use eager RDMA)",
@@ -411,7 +429,7 @@ int btl_openib_register_mca_params(void)
     mca_btl_openib_component.max_eager_rdma = (int32_t) ival;
 
     CHECK(reg_int("eager_rdma_num", NULL, "Number of RDMA buffers to allocate "
-                  "for small messages"
+                  "for small messages "
                   "(must be >= 1)",
                   16, &ival, REGINT_GE_ONE));
     mca_btl_openib_component.eager_rdma_num = (int32_t) (ival + 1);
@@ -427,33 +445,52 @@ int btl_openib_register_mca_params(void)
                   0, &ival, REGINT_GE_ZERO));
     mca_btl_openib_component.max_lmc = (uint32_t) ival;
 
-#if OMPI_HAVE_THREADS
-    CHECK(reg_int("enable_apm_over_lmc", NULL, "Maximum number of alterative paths for each device port "
-                  "(must be >= -1, where 0 = disable apm, -1 = all availible alternative paths )",
+#if OPAL_HAVE_THREADS
+    CHECK(reg_int("enable_apm_over_lmc", NULL, "Maximum number of alternative paths for each device port "
+                  "(must be >= -1, where 0 = disable apm, -1 = all available alternative paths )",
                   0, &ival, REGINT_NEG_ONE_OK|REGINT_GE_ZERO));
     mca_btl_openib_component.apm_lmc = (uint32_t) ival;
-    CHECK(reg_int("enable_apm_over_ports", NULL, "Enable alterative path migration (APM) over different ports of the same device "
-                  "(must be >= 0, where 0 = disable APM over ports , 1 = enable APM over ports of the same device)",
-                  0, &ival, REGINT_GE_ZERO));
-    mca_btl_openib_component.apm_ports = (uint32_t) ival;
-
-    CHECK(reg_int("enable_apm_over_lmc", NULL, "Maximum number of alterative paths for each device port "
-                  "(must be >= -1, where 0 = disable APM, -1 = all availible alternative paths)",
-                  0, &ival, REGINT_NEG_ONE_OK|REGINT_GE_ZERO));
-    mca_btl_openib_component.apm_lmc = (uint32_t) ival;
-    CHECK(reg_int("enable_apm_over_ports", NULL, "Enable alterative path migration (APM) over different ports of the same device "
+    CHECK(reg_int("enable_apm_over_ports", NULL, "Enable alternative path migration (APM) over different ports of the same device "
                   "(must be >= 0, where 0 = disable APM over ports, 1 = enable APM over ports of the same device)",
                   0, &ival, REGINT_GE_ZERO));
     mca_btl_openib_component.apm_ports = (uint32_t) ival;
 
     CHECK(reg_int("use_async_event_thread", NULL,
-                "If nonzero, use the thread that will handle InfiniBand asyncihronous events ",
+                "If nonzero, use the thread that will handle InfiniBand asynchronous events",
                 1, &ival, 0));
     mca_btl_openib_component.use_async_event_thread = (0 != ival);
+
+#if BTL_OPENIB_FAILOVER_ENABLED
+    /* failover specific output */
+    CHECK(reg_int("verbose_failover", NULL,
+                  "Output some verbose OpenIB BTL failover information "
+                  "(0 = no output, nonzero = output)", 0, &ival, 0));
+    mca_btl_openib_component.verbose_failover = opal_output_open(NULL);
+    opal_output_set_verbosity(mca_btl_openib_component.verbose_failover, ival);
+
+    CHECK(reg_int("port_error_failover", NULL,
+                  "If nonzero, asynchronous port errors will trigger failover",
+                  0, &ival, 0));
+    mca_btl_openib_component.port_error_failover = (0 != ival);
+
+    /* Make non writeable parameter that indicates failover is configured in. */
+    tmp = mca_base_param_reg_int(&mca_btl_openib_component.super.btl_version,
+                                 "failover_enabled",
+                                 "openib failover is configured: run with bfo PML to support failover between openib BTLs",
+                                 false, true,
+                                 1, NULL);
+#endif
+
+    CHECK(reg_int("enable_srq_resize", NULL,
+                  "Enable/Disable on demand SRQ resize. "
+                  "(0 = without resizing, nonzero = with resizing)", 1, &ival, 0));
+    mca_btl_openib_component.enable_srq_resize = (0 != ival);
+#else
+    mca_btl_openib_component.enable_srq_resize = 0;
 #endif
 
     CHECK(reg_int("buffer_alignment", NULL,
-                  "Prefered communication buffer alignment, in bytes "
+                  "Preferred communication buffer alignment, in bytes "
                   "(must be > 0 and power of two)",
                   64, &ival, REGINT_GE_ZERO));
     if(ival <= 1 || (ival & (ival - 1))) {
@@ -465,19 +502,19 @@ int btl_openib_register_mca_params(void)
     }
 
     CHECK(reg_int("use_message_coalescing", NULL,
-                  "Use message coalescing", 1, &ival, 0));
+                  "If nonzero, use message coalescing", 1, &ival, 0));
     mca_btl_openib_component.use_message_coalescing = (0 != ival);
 
     CHECK(reg_int("cq_poll_ratio", NULL,
-                  "how often poll high priority CQ versus low priority CQ",
+                  "How often to poll high priority CQ versus low priority CQ",
                   100, &ival, REGINT_GE_ONE));
     mca_btl_openib_component.cq_poll_ratio = (uint32_t)ival;
     CHECK(reg_int("eager_rdma_poll_ratio", NULL,
-                  "how often poll eager RDMA channel versus CQ",
+                  "How often to poll eager RDMA channel versus CQ",
                   100, &ival, REGINT_GE_ONE));
     mca_btl_openib_component.eager_rdma_poll_ratio = (uint32_t)ival;
     CHECK(reg_int("hp_cq_poll_per_progress", NULL,
-                  "max number of completion events to process for each call "
+                  "Max number of completion events to process for each call "
                   "of BTL progress engine",
                   10, &ival, REGINT_GE_ONE));
     mca_btl_openib_component.cq_poll_progress = (uint32_t)ival;
@@ -485,7 +522,8 @@ int btl_openib_register_mca_params(void)
     /* Info only */
     mca_base_param_reg_int(&mca_btl_openib_component.super.btl_version,
                            "have_fork_support",
-                           "Whether the OpenFabrics stack supports applications that invoke the \"fork()\" system call or not (0 = no, 1 = yes).  Note that this value does NOT indicate whether the system being run on supports \"fork()\" with OpenFabrics applications or not.",
+                           "Whether the OpenFabrics stack supports applications that invoke the \"fork()\" system call or not (0 = no, 1 = yes). "
+                           "Note that this value does NOT indicate whether the system being run on supports \"fork()\" with OpenFabrics applications or not.",
                            false, true,
                            OMPI_HAVE_IBV_FORK_INIT ? 1 : 0,
                            NULL);
@@ -500,8 +538,12 @@ int btl_openib_register_mca_params(void)
     mca_btl_openib_module.super.btl_min_rdma_pipeline_size = 256 * 1024;
     mca_btl_openib_module.super.btl_flags = MCA_BTL_FLAGS_RDMA |
         MCA_BTL_FLAGS_NEED_ACK | MCA_BTL_FLAGS_NEED_CSUM | MCA_BTL_FLAGS_HETEROGENEOUS_RDMA;
-    mca_btl_openib_module.super.btl_bandwidth = 800;
-    mca_btl_openib_module.super.btl_latency = 10;
+#if BTL_OPENIB_FAILOVER_ENABLED
+    mca_btl_openib_module.super.btl_flags |= MCA_BTL_FLAGS_FAILOVER_SUPPORT;
+#endif
+    /* Default to bandwidth auto-detection */
+    mca_btl_openib_module.super.btl_bandwidth = 0;
+    mca_btl_openib_module.super.btl_latency = 4;
     CHECK(mca_btl_base_param_register(
             &mca_btl_openib_component.super.btl_version,
             &mca_btl_openib_module.super));
@@ -534,7 +576,7 @@ int btl_openib_register_mca_params(void)
     }
 
     CHECK(reg_string("receive_queues", NULL,
-                     "Colon-delimited, comma delimited list of receive queues: P,4096,8,6,4:P,32768,8,6,4",
+                     "Colon-delimited, comma-delimited list of receive queues: P,4096,8,6,4:P,32768,8,6,4",
                      default_qps, &mca_btl_openib_component.receive_queues, 
                      0));
     mca_btl_openib_component.receive_queues_source = 
@@ -562,6 +604,11 @@ int btl_openib_register_mca_params(void)
                      NULL, &mca_btl_openib_component.ipaddr_exclude,
                      0));
 
+    CHECK(reg_int("gid_index", NULL,
+                  "GID index to use on verbs device ports",
+                  0, &mca_btl_openib_component.gid_index,
+                  REGINT_GE_ZERO));
+
     /* Register any MCA params for the connect pseudo-components */
     if (OMPI_SUCCESS == ret) {
         ret = ompi_btl_openib_connect_base_register();
