@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -11,8 +11,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Voltaire. All rights reserved.
- * Copyright (c) 2010      Los Alamos National Security, LLC.
- *                         All rights reserved.
+ * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,7 +24,29 @@
 #ifndef MCA_BTL_SM_H
 #define MCA_BTL_SM_H
 
+#include "ompi_config.h"
+#include <stddef.h>
 #include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif  /* HAVE_STDINT_H */
+#ifdef HAVE_SCHED_H
+#include <sched.h>
+#endif  /* HAVE_SCHED_H */
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif  /* HAVE_SYS_IOCTL_H */
+#ifdef HAVE_SYS_MMAN_H
+#include <sys/mman.h>
+#endif  /* HAVE_SYS_MMAN_H */
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif  /* HAVE_FCNTL_H */
+#if OMPI_BTL_SM_HAVE_KNEM
+#include "knem_io.h"
+#endif  /* OMPI_BTL_SM_HAVE_KNEM */
+
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif  /* HAVE_SYS_TYPES_H */
@@ -40,20 +61,15 @@
 #endif  /* HAVE_UNISTD_H */
 #include "opal/class/opal_free_list.h"
 #include "ompi/class/ompi_free_list.h"
-#include "ompi/class/ompi_bitmap.h"
 #include "opal/event/event.h"
-#include "ompi/mca/pml/pml.h"
+#include "opal/threads/threads.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/base.h"
 
 #include "ompi/mca/mpool/mpool.h"
 #include "ompi/mca/common/sm/common_sm_mmap.h"
 
-#include "opal/mca/maffinity/base/base.h"
-
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
 /*
  * Shared Memory FIFOs
@@ -84,32 +100,43 @@ extern "C" {
  */
 
 #define SM_FIFO_FREE  (void *) (-2)
+/* We can't use opal_cache_line_size here because we need a
+   compile-time constant for padding the struct.  We can't really have
+   a compile-time constant that is portable, either (e.g., compile on
+   one machine and run on another).  So just use a big enough cache
+   line that should hopefully be good in most places. */
+#define SM_CACHE_LINE_PAD 128
 
-struct mca_btl_sm_fifo_t {
+struct sm_fifo_t {
     /* This queue pointer is used only by the heads. */
-    volatile void **queue;           char pad0[CACHE_LINE_SIZE - sizeof(void **)           ];
+    volatile void **queue;           
+    char pad0[SM_CACHE_LINE_PAD - sizeof(void **)];
     /* This lock is used by the heads. */
-    opal_atomic_lock_t head_lock;    char pad1[CACHE_LINE_SIZE - sizeof(opal_atomic_lock_t)];
+    opal_atomic_lock_t head_lock;    
+    char pad1[SM_CACHE_LINE_PAD - sizeof(opal_atomic_lock_t)];
     /* This index is used by the head holding the head lock. */
-    volatile int head;               char pad2[CACHE_LINE_SIZE - sizeof(int)               ];
+    volatile int head;               
+    char pad2[SM_CACHE_LINE_PAD - sizeof(int)];
     /* This mask is used "read only" by all processes. */
-    unsigned int mask;               char pad3[CACHE_LINE_SIZE - sizeof(int)               ];
+    unsigned int mask;               
+    char pad3[SM_CACHE_LINE_PAD - sizeof(int)];
     /* The following are used only by the tail. */
     volatile void **queue_recv;
     opal_atomic_lock_t tail_lock;
     volatile int tail;
     int num_to_clear;
-    int lazy_free;                   char pad4[CACHE_LINE_SIZE - sizeof(void **)
-                                                               - sizeof(opal_atomic_lock_t)
-                                                               - sizeof(int) * 3           ];
+    int lazy_free;                   
+    char pad4[SM_CACHE_LINE_PAD - sizeof(void **) -
+              sizeof(opal_atomic_lock_t) -
+              sizeof(int) * 3];
 };
-typedef struct mca_btl_sm_fifo_t mca_btl_sm_fifo_t;
+typedef struct sm_fifo_t sm_fifo_t;
 
 /*
  * Shared Memory resource managment
  */
 
-#if OMPI_ENABLE_PROGRESS_THREADS == 1
+#if OPAL_ENABLE_PROGRESS_THREADS == 1
 #define DATA (char)0
 #define DONE (char)1
 #endif
@@ -138,10 +165,10 @@ struct mca_btl_sm_component_t {
     mca_common_sm_mmap_t *mmap_file;   /**< description of mmap'ed file */
     mca_common_sm_file_header_t *sm_ctl_header;  /* control header in
                                                     shared memory */
-    mca_btl_sm_fifo_t **shm_fifo;      /**< pointer to fifo 2D array in shared memory */
+    volatile sm_fifo_t **shm_fifo;     /**< pointer to fifo 2D array in shared memory */
     char **shm_bases;                  /**< pointer to base pointers in shared memory */
     uint16_t *shm_mem_nodes;           /**< pointer to mem noded in shared memory */
-    mca_btl_sm_fifo_t **fifo;          /**< cached copy of the pointer to the 2D
+    sm_fifo_t **fifo;                  /**< cached copy of the pointer to the 2D
                                           fifo array.  The address in the shared
                                           memory segment sm_ctl_header is a relative,
                                           but this one, in process private memory, is
@@ -150,13 +177,12 @@ struct mca_btl_sm_component_t {
     size_t fifo_size;                  /**< number of FIFO queue entries */
     size_t fifo_lazy_free;             /**< number of reads before lazy fifo free is triggered */
     int nfifos;                        /**< number of FIFOs per receiver */
-    ptrdiff_t *sm_offset;              /**< offset to be applied to shared memory
-                                          addresses, per local process value */
     int32_t num_smp_procs;             /**< current number of smp procs on this host */
     int32_t my_smp_rank;               /**< My SMP process rank.  Used for accessing
                                         *   SMP specfic data structures. */
     ompi_free_list_t sm_frags_eager;   /**< free list of sm first */
     ompi_free_list_t sm_frags_max;     /**< free list of sm second */
+    ompi_free_list_t sm_frags_user;
     ompi_free_list_t sm_first_frags_to_progress;  /**< list of first
                                                     fragments that are
                                                     awaiting resources */
@@ -168,21 +194,83 @@ struct mca_btl_sm_component_t {
     int mem_node;
     int num_mem_nodes;
     
-#if OMPI_ENABLE_PROGRESS_THREADS == 1
+#if OPAL_ENABLE_PROGRESS_THREADS == 1
     char sm_fifo_path[PATH_MAX];   /**< path to fifo used to signal this process */
     int  sm_fifo_fd;               /**< file descriptor corresponding to opened fifo */
     opal_thread_t sm_fifo_thread;
 #endif
+    struct mca_btl_sm_t      **sm_btls;
+    struct mca_btl_sm_frag_t **table;
+    size_t sm_num_btls;
+    size_t sm_max_btls;
+
+#if OMPI_BTL_SM_HAVE_KNEM
+    /* Knem capabilities info */
+    struct knem_cmd_info knem_info;
+#endif
+
+    /** MCA: should we be using knem or not?  neg=try but continue if
+        not available, 0=don't try, 1=try and fail if not available */
+    int use_knem;
+
+    /** MCA: minimal message size (bytes) to offload on DMA engine
+        when using knem */
+    uint32_t knem_dma_min;
+
+    /** MCA: how many simultaneous ongoing knem operations to
+        support */
+    int knem_max_simultaneous;
+
+    /** If we want DMA and DMA is supported, this will be loaded with
+        KNEM_FLAG_DMA.  Otherwise, it'll be 0. */
+    int knem_dma_flag;
 };
 typedef struct mca_btl_sm_component_t mca_btl_sm_component_t;
 OMPI_MODULE_DECLSPEC extern mca_btl_sm_component_t mca_btl_sm_component;
 
-struct mca_btl_sm_pending_send_item_t
+/**
+ * SM BTL Interface
+ */
+struct mca_btl_sm_t {
+    mca_btl_base_module_t  super;       /**< base BTL interface */
+    bool btl_inited;  /**< flag indicating if btl has been inited */
+    mca_btl_base_module_error_cb_fn_t error_cb;
+
+#if OMPI_BTL_SM_HAVE_KNEM
+
+    /* File descriptor for knem */
+    int knem_fd;
+
+    /* Array of knem status items for non-blocking knem requests */
+    knem_status_t *knem_status_array;
+
+    /* Array of fragments currently being moved by knem non-blocking
+       operations */
+    struct mca_btl_sm_frag_t **knem_frag_array;
+
+    /* First free/available location in knem_status_array */
+    int knem_status_first_avail;
+
+    /* First currently-being used location in the knem_status_array */
+    int knem_status_first_used;
+
+    /* Number of status items currently in use */
+    int knem_status_num_used;
+#endif
+};
+typedef struct mca_btl_sm_t mca_btl_sm_t;
+OMPI_MODULE_DECLSPEC extern mca_btl_sm_t mca_btl_sm;
+
+
+
+
+
+struct btl_sm_pending_send_item_t
 {
     opal_free_list_item_t super;
     void *data;
 };
-typedef struct mca_btl_sm_pending_send_item_t mca_btl_sm_pending_send_item_t;
+typedef struct btl_sm_pending_send_item_t btl_sm_pending_send_item_t;
 
 /***
  * FIFO support for sm BTL.
@@ -199,12 +287,8 @@ typedef struct mca_btl_sm_pending_send_item_t mca_btl_sm_pending_send_item_t;
 #define VIRTUAL2RELATIVE(VADDR ) ((long)(VADDR)  - (long)mca_btl_sm_component.shm_bases[mca_btl_sm_component.my_smp_rank])
 #define RELATIVE2VIRTUAL(OFFSET) ((long)(OFFSET) + (long)mca_btl_sm_component.shm_bases[mca_btl_sm_component.my_smp_rank])
 
-/* ================================================== */
-/* ================================================== */
-/* ================================================== */
-
 static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool,
-                               mca_btl_sm_fifo_t *fifo, int lazy_free)
+                               sm_fifo_t *fifo, int lazy_free)
 {
     int i, qsize;
 
@@ -215,7 +299,7 @@ static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool,
 
     /* allocate the queue in the receiver's address space */
     fifo->queue_recv = (volatile void **)mpool->mpool_alloc(
-            mpool, sizeof(void *) * qsize, CACHE_LINE_SIZE, 0, NULL);
+            mpool, sizeof(void *) * qsize, opal_cache_line_size, 0, NULL);
     if(NULL == fifo->queue_recv) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
@@ -244,7 +328,7 @@ static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool,
 }
 
 
-static inline int sm_fifo_write(void *value, mca_btl_sm_fifo_t *fifo)
+static inline int sm_fifo_write(void *value, sm_fifo_t *fifo)
 {
     volatile void **q = (volatile void **) RELATIVE2VIRTUAL(fifo->queue);
 
@@ -255,13 +339,13 @@ static inline int sm_fifo_write(void *value, mca_btl_sm_fifo_t *fifo)
 
     /* otherwise, write to the slot and advance the head index */
     q[fifo->head] = value;
-    opal_atomic_wmb();
+    opal_atomic_wmb(); 
     fifo->head = (fifo->head + 1) & fifo->mask;
     return OMPI_SUCCESS;
 }
 
 
-static inline void *sm_fifo_read(mca_btl_sm_fifo_t *fifo)
+static inline void *sm_fifo_read(sm_fifo_t *fifo)
 {
     void *value;
 
@@ -321,17 +405,7 @@ extern mca_btl_base_module_t** mca_btl_sm_component_init(
  */
 extern int mca_btl_sm_component_progress(void);
 
-/**
- * SM BTL Interface
- */
-struct mca_btl_sm_t {
-    mca_btl_base_module_t  super;       /**< base BTL interface */
-    bool btl_inited;  /**< flag indicating if btl has been inited */
-    mca_btl_base_module_error_cb_fn_t error_cb;
-};
-typedef struct mca_btl_sm_t mca_btl_sm_t;
 
-extern mca_btl_sm_t mca_btl_sm;
 
 /**
  * Register a callback function that is called on error..
@@ -376,7 +450,7 @@ extern int mca_btl_sm_add_procs(
     size_t nprocs,
     struct ompi_proc_t **procs,
     struct mca_btl_base_endpoint_t** peers,
-    struct ompi_bitmap_t* reachability
+    struct opal_bitmap_t* reachability
 );
 
 
@@ -433,7 +507,7 @@ struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_src(
     struct mca_btl_base_module_t* btl,
     struct mca_btl_base_endpoint_t* endpoint,
     mca_mpool_base_registration_t* registration,
-    struct ompi_convertor_t* convertor,
+    struct opal_convertor_t* convertor,
     uint8_t order,
     size_t reserve,
     size_t* size,
@@ -449,7 +523,7 @@ struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_src(
  */
 extern int mca_btl_sm_sendi( struct mca_btl_base_module_t* btl,
                              struct mca_btl_base_endpoint_t* endpoint,
-                             struct ompi_convertor_t* convertor,
+                             struct opal_convertor_t* convertor,
                              void* header,
                              size_t header_size,
                              size_t payload_size,
@@ -471,6 +545,33 @@ extern int mca_btl_sm_send(
     mca_btl_base_tag_t tag
 );
 
+#if OMPI_BTL_SM_HAVE_KNEM
+/*
+ * Synchronous knem get
+ */
+extern int mca_btl_sm_get_sync(
+		struct mca_btl_base_module_t* btl,
+		struct mca_btl_base_endpoint_t* endpoint,
+		struct mca_btl_base_descriptor_t* des );
+/*
+ * Asynchronous knem get
+ */
+extern int mca_btl_sm_get_async(
+		struct mca_btl_base_module_t* btl,
+		struct mca_btl_base_endpoint_t* endpoint,
+		struct mca_btl_base_descriptor_t* des );
+
+extern struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_dst(
+		struct mca_btl_base_module_t* btl,
+		struct mca_btl_base_endpoint_t* endpoint,
+		struct mca_mpool_base_registration_t* registration,
+		struct opal_convertor_t* convertor,
+		uint8_t order,
+		size_t reserve,
+		size_t* size,
+		uint32_t flags);
+#endif
+
 /**
  * Fault Tolerance Event Notification Function
  * @param state Checkpoint Stae
@@ -478,11 +579,11 @@ extern int mca_btl_sm_send(
  */
 int mca_btl_sm_ft_event(int state);
 
-#if OMPI_ENABLE_PROGRESS_THREADS == 1
+#if OPAL_ENABLE_PROGRESS_THREADS == 1
 void mca_btl_sm_component_event_thread(opal_object_t*);
 #endif
 
-#if OMPI_ENABLE_PROGRESS_THREADS == 1
+#if OPAL_ENABLE_PROGRESS_THREADS == 1
 #define MCA_BTL_SM_SIGNAL_PEER(peer) \
 { \
     unsigned char cmd = DATA; \
@@ -494,9 +595,7 @@ void mca_btl_sm_component_event_thread(opal_object_t*);
 #define MCA_BTL_SM_SIGNAL_PEER(peer)
 #endif
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif
 
