@@ -25,6 +25,7 @@
 
 #include "mpi.h"
 #include "opal/mca/mca.h"
+#include "opal/datatype/opal_convertor.h"
 #include "orte/types.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/common/sm/common_sm_mmap.h"
@@ -367,7 +368,7 @@ extern uint32_t mca_coll_sm_one;
         (index)->mcbmi_data + \
         ((rank) * mca_coll_sm_component.sm_fragment_size); \
     (iov).iov_len = (max_data); \
-    ompi_convertor_pack(&(convertor), &(iov), &mca_coll_sm_one, \
+    opal_convertor_pack(&(convertor), &(iov), &mca_coll_sm_one, \
                         &(max_data) )
 
 /**
@@ -378,7 +379,7 @@ extern uint32_t mca_coll_sm_one;
     (iov).iov_base = (((char*) (index)->mcbmi_data) + \
                        ((src_rank) * (mca_coll_sm_component.sm_fragment_size))); \
     (iov).iov_len = (max_data); \
-    ompi_convertor_unpack(&(convertor), &(iov), &mca_coll_sm_one, \
+    opal_convertor_unpack(&(convertor), &(iov), &mca_coll_sm_one, \
                           &(max_data) )
 
 /**
