@@ -30,6 +30,9 @@
 #include "ompi_config.h"
 
 #include <stdio.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 #ifdef HAVE_SCHED_H
 #include <sched.h>
 #endif
@@ -143,7 +146,7 @@ int mca_coll_sm_init_query(bool enable_progress_threads,
     for (i = 0; i < size; ++i) {
         if (procs[i] != my_proc &&
             procs[i]->proc_name.jobid == my_proc->proc_name.jobid &&
-            procs[i]->proc_flags & OMPI_PROC_FLAG_LOCAL) {
+            OPAL_PROC_ON_LOCAL_NODE(procs[i]->proc_flags)) {
             break;
         }
     }
@@ -497,7 +500,7 @@ static bool have_local_peers(ompi_group_t *group, size_t size)
 
     for (i = 0; i < size; ++i) {
         proc = ompi_group_peer_lookup(group,i);
-        if (0 == (proc->proc_flags & OMPI_PROC_FLAG_LOCAL)) {
+        if (!OPAL_PROC_ON_LOCAL_NODE(proc->proc_flags)) {
             return false;
         }
     }
