@@ -20,7 +20,7 @@
 
 #include "mpi.h"
 #include "ompi/constants.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/coll_tags.h"
@@ -58,8 +58,8 @@ ompi_coll_tuned_bcast_intra_generic( void* buffer,
     rank = ompi_comm_rank(comm);
     assert( size > 1 );
 
-    ompi_ddt_get_extent (datatype, &lb, &extent);
-    ompi_ddt_type_size( datatype, &type_size );
+    ompi_datatype_get_extent (datatype, &lb, &extent);
+    ompi_datatype_type_size( datatype, &type_size );
     num_segments = (original_count + count_by_segment - 1) / count_by_segment;
     realsegsize = count_by_segment * extent;
     
@@ -272,7 +272,7 @@ ompi_coll_tuned_bcast_intra_bintree ( void* buffer,
     /**
      * Determine number of elements sent per operation.
      */
-    ompi_ddt_type_size( datatype, &typelng );
+    ompi_datatype_type_size( datatype, &typelng );
     COLL_TUNED_COMPUTED_SEGCOUNT( segsize, typelng, segcount );
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:bcast_intra_binary rank %d ss %5d typelng %lu segcount %d",
@@ -301,7 +301,7 @@ ompi_coll_tuned_bcast_intra_pipeline( void* buffer,
     /**
      * Determine number of elements sent per operation.
      */
-    ompi_ddt_type_size( datatype, &typelng );
+    ompi_datatype_type_size( datatype, &typelng );
     COLL_TUNED_COMPUTED_SEGCOUNT( segsize, typelng, segcount );
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:bcast_intra_pipeline rank %d ss %5d typelng %lu segcount %d",
@@ -330,7 +330,7 @@ ompi_coll_tuned_bcast_intra_chain( void* buffer,
     /**
      * Determine number of elements sent per operation.
      */
-    ompi_ddt_type_size( datatype, &typelng );
+    ompi_datatype_type_size( datatype, &typelng );
     COLL_TUNED_COMPUTED_SEGCOUNT( segsize, typelng, segcount );
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:bcast_intra_chain rank %d fo %d ss %5d typelng %lu segcount %d",
@@ -359,7 +359,7 @@ ompi_coll_tuned_bcast_intra_binomial( void* buffer,
     /**
      * Determine number of elements sent per operation.
      */
-    ompi_ddt_type_size( datatype, &typelng );
+    ompi_datatype_type_size( datatype, &typelng );
     COLL_TUNED_COMPUTED_SEGCOUNT( segsize, typelng, segcount );
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:bcast_intra_binomial rank %d ss %5d typelng %lu segcount %d",
@@ -407,14 +407,14 @@ ompi_coll_tuned_bcast_intra_split_bintree ( void* buffer,
     COLL_TUNED_UPDATE_BINTREE( comm, tuned_module, root );
     tree = data->cached_bintree;
 
-    err = ompi_ddt_type_size( datatype, &type_size );
+    err = ompi_datatype_type_size( datatype, &type_size );
 
     /* Determine number of segments and number of elements per segment */
     counts[0] = count/2;
     if (count % 2 != 0) counts[0]++;
     counts[1] = count - counts[0];
     if ( segsize > 0 ) {
-        /* Note that ompi_ddt_type_size() will never return a negative
+        /* Note that ompi_datatype_type_size() will never return a negative
            value in typelng; it returns an int [vs. an unsigned type]
            because of the MPI spec. */
     	if (segsize < ((uint32_t) type_size)) {
@@ -441,7 +441,7 @@ ompi_coll_tuned_bcast_intra_split_bintree ( void* buffer,
 						    segsize, 1 ));
     }
 
-    err = ompi_ddt_get_extent (datatype, &lb, &type_extent);
+    err = ompi_datatype_get_extent (datatype, &lb, &type_extent);
     
     /* Determine real segment size */
     realsegsize[0] = segcount[0] * type_extent;
@@ -736,8 +736,11 @@ int ompi_coll_tuned_bcast_intra_check_forced_init (coll_tuned_force_algorithm_mc
                                  "bcast_algorithm",
                                  "Which bcast algorithm is used. Can be locked down to choice of: 0 ignore, 1 basic linear, 2 chain, 3: pipeline, 4: split binary tree, 5: binary tree, 6: binomial tree.",
                                  false, false, 0, NULL);
+    if (mca_param_indices->algorithm_param_index < 0) {
+        return mca_param_indices->algorithm_param_index;
+    }
     mca_base_param_lookup_int(mca_param_indices->algorithm_param_index, &(requested_alg));
-    if( requested_alg > max_alg ) {
+    if( 0 > requested_alg || requested_alg > max_alg ) {
         if( 0 == ompi_comm_rank( MPI_COMM_WORLD ) ) {
             opal_output( 0, "Broadcast algorithm #%d is not available (range [0..%d]). Switching back to ignore(0)\n",
                          requested_alg, max_alg );
