@@ -20,14 +20,12 @@
 #include "ompi_config.h"
 
 #include "mpi.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/coll_tags.h"
 #include "ompi/op/op.h"
 #include "coll_tuned.h"
-#include "ompi/mca/pml/pml.h"
-#include "opal/util/bit_ops.h"
 
 
 /*
@@ -56,7 +54,7 @@ ompi_coll_tuned_allreduce_intra_dec_fixed (void *sbuf, void *rbuf, int count,
      * can handle both commutative and non-commutative operations.
      * Ring algorithm does not support non-commutative operations.
      */
-    ompi_ddt_type_size(dtype, &dsize);
+    ompi_datatype_type_size(dtype, &dsize);
     block_dsize = dsize * count;
 
     if (block_dsize < intermediate_message) {
@@ -116,7 +114,7 @@ int ompi_coll_tuned_alltoall_intra_dec_fixed(void *sbuf, int scount,
        the University of Tennessee (2GB MX) up to 64 nodes.
        Has better performance for messages of intermediate sizes than the old one */
     /* determine block size */
-    ompi_ddt_type_size(sdtype, &dsize);
+    ompi_datatype_type_size(sdtype, &dsize);
     block_dsize = dsize * scount;
 
     if ((block_dsize < 200) && (communicator_size > 12)) {
@@ -138,7 +136,7 @@ int ompi_coll_tuned_alltoall_intra_dec_fixed(void *sbuf, int scount,
     /* previous decision */
 
     /* else we need data size for decision function */
-    ompi_ddt_type_size(sdtype, &dsize);
+    ompi_datatype_type_size(sdtype, &dsize);
     total_dsize = dsize * scount * communicator_size;   /* needed for decision */
 
     OPAL_OUTPUT((ompi_coll_tuned_stream, "ompi_coll_tuned_alltoall_intra_dec_fixed rank %d com_size %d msg_length %ld",
@@ -241,7 +239,7 @@ int ompi_coll_tuned_bcast_intra_dec_fixed(void *buff, int count,
     communicator_size = ompi_comm_size(comm);
 
     /* else we need data size for decision function */
-    ompi_ddt_type_size(datatype, &dsize);
+    ompi_datatype_type_size(datatype, &dsize);
     message_size = dsize * (unsigned long)count;   /* needed for decision */
 
     OPAL_OUTPUT((ompi_coll_tuned_stream, "ompi_coll_tuned_bcast_intra_dec_fixed"
@@ -360,7 +358,7 @@ int ompi_coll_tuned_reduce_intra_dec_fixed( void *sendbuf, void *recvbuf,
     communicator_size = ompi_comm_size(comm);
 
     /* need data size for decision function */
-    ompi_ddt_type_size(datatype, &dsize);
+    ompi_datatype_type_size(datatype, &dsize);
     message_size = dsize * count;   /* needed for decision */
 
     /**
@@ -401,8 +399,8 @@ int ompi_coll_tuned_reduce_intra_dec_fixed( void *sendbuf, void *recvbuf,
     } else if (communicator_size > (a3 * message_size + b3)) {
         /* Binary_32K */
         segsize = 32*1024;
-        return ompi_coll_tuned_reduce_intra_pipeline (sendbuf, recvbuf, count, datatype, op, root, comm, module,
-						      segsize, max_requests);
+        return ompi_coll_tuned_reduce_intra_binary( sendbuf, recvbuf, count, datatype, op, root,
+                                                    comm, module, segsize, max_requests);
     }
     if (communicator_size > (a4 * message_size + b4)) {
         /* Pipeline_32K */
@@ -473,7 +471,7 @@ int ompi_coll_tuned_reduce_scatter_intra_dec_fixed( void *sbuf, void *rbuf,
 
     comm_size = ompi_comm_size(comm);
     /* We need data size for decision function */
-    ompi_ddt_type_size(dtype, &dsize);
+    ompi_datatype_type_size(dtype, &dsize);
     total_message_size = 0;
     for (i = 0; i < comm_size; i++) { 
         total_message_size += rcounts[i];
@@ -534,9 +532,9 @@ int ompi_coll_tuned_allgather_intra_dec_fixed(void *sbuf, int scount,
                                                           comm, module);
     }
 
-   /* Determine complete data size */
-   ompi_ddt_type_size(sdtype, &dsize);
-   total_dsize = dsize * scount * communicator_size;   
+    /* Determine complete data size */
+    ompi_datatype_type_size(sdtype, &dsize);
+    total_dsize = dsize * scount * communicator_size;   
    
     OPAL_OUTPUT((ompi_coll_tuned_stream, "ompi_coll_tuned_allgather_intra_dec_fixed"
                  " rank %d com_size %d msg_length %lu",
@@ -630,7 +628,7 @@ int ompi_coll_tuned_allgatherv_intra_dec_fixed(void *sbuf, int scount,
     }
     
     /* Determine complete data size */
-    ompi_ddt_type_size(sdtype, &dsize);
+    ompi_datatype_type_size(sdtype, &dsize);
     total_dsize = 0;
     for (i = 0; i < communicator_size; i++) {
         total_dsize += dsize * rcounts[i];
@@ -697,10 +695,10 @@ int ompi_coll_tuned_gather_intra_dec_fixed(void *sbuf, int scount,
 
     /* Determine block size */
     if (rank == root) {
-        ompi_ddt_type_size(rdtype, &dsize);
+        ompi_datatype_type_size(rdtype, &dsize);
         block_size = dsize * rcount;
     } else {
-        ompi_ddt_type_size(sdtype, &dsize);
+        ompi_datatype_type_size(sdtype, &dsize);
         block_size = dsize * scount;
     }
 
@@ -758,10 +756,10 @@ int ompi_coll_tuned_scatter_intra_dec_fixed(void *sbuf, int scount,
     rank = ompi_comm_rank(comm);
     /* Determine block size */
     if (root == rank) {
-        ompi_ddt_type_size(sdtype, &dsize);
+        ompi_datatype_type_size(sdtype, &dsize);
         block_size = dsize * scount;
     } else {
-        ompi_ddt_type_size(rdtype, &dsize);
+        ompi_datatype_type_size(rdtype, &dsize);
         block_size = dsize * rcount;
     } 
 
