@@ -5,7 +5,7 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2010 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -44,18 +44,15 @@
 #include <sys/mman.h>
 #endif
 
-#include "opal/runtime/opal.h"
-#include "opal/util/basename.h"
-#include "orte/util/show_help.h"
+#include "opal/util/output.h"
 #include "opal/util/path.h"
 #include "opal/align.h"
 #include "opal/types.h"
 #include "opal/threads/mutex.h"
 
-#include "orte/util/proc_info.h"
-#include "orte/mca/rml/rml_types.h"
-#include "orte/mca/rml/base/base.h"
+#include "orte/mca/rml/rml.h"
 #include "orte/util/name_fns.h"
+#include "orte/util/show_help.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/mca/errmgr/errmgr.h"
 
@@ -92,7 +89,7 @@ static bool pending_rml_msgs_init = false;
  */
 typedef struct {
     opal_list_item_t super;
-    char file_name[OMPI_PATH_MAX];
+    char file_name[OPAL_PATH_MAX];
     int sm_file_inited;
 } pending_rml_msg_t;
 
@@ -121,7 +118,7 @@ static mca_common_sm_mmap_t* create_map(int fd, size_t size, char *file_name,
 
     /* set up the map object */
     map = OBJ_NEW(mca_common_sm_mmap_t);
-    strncpy(map->map_path, file_name, OMPI_PATH_MAX);
+    strncpy(map->map_path, file_name, OPAL_PATH_MAX);
     /* the first entry in the file is the control structure. The first
        entry in the control structure is an mca_common_sm_file_header_t
        element */
@@ -164,7 +161,7 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init(ompi_proc_t **procs,
     int rc = 0, sm_file_inited = 0, num_local_procs;
     struct iovec iov[3];
     int sm_file_created = OMPI_RML_TAG_SM_BACK_FILE_CREATED;
-    char filename_to_send[OMPI_PATH_MAX];
+    char filename_to_send[OPAL_PATH_MAX];
     opal_list_item_t *item;
     pending_rml_msg_t *rml_msg;
     ompi_proc_t *temp_proc;
@@ -175,7 +172,7 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init(ompi_proc_t **procs,
        lowest name.  Ensure that procs[0] is the lowest named
        process. */
     for (num_local_procs = p = 0; p < num_procs; p++) {
-        if (procs[p]->proc_flags & OMPI_PROC_FLAG_LOCAL) {
+        if (OPAL_PROC_ON_LOCAL_NODE(procs[p]->proc_flags)) {
             /* If we don't have a lowest, save the first one */
             if (!found_lowest) {
                 procs[0] = procs[p];
@@ -208,7 +205,7 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init(ompi_proc_t **procs,
     iov[0].iov_len = sizeof(sm_file_created);
     memset(filename_to_send, 0, sizeof(filename_to_send));
     strncpy(filename_to_send, file_name, sizeof(filename_to_send) - 1);
-    iov[1].iov_base = (ompi_iov_base_ptr_t)filename_to_send;
+    iov[1].iov_base = filename_to_send;
     iov[1].iov_len = sizeof(filename_to_send);
     iov[2].iov_base = (ompi_iov_base_ptr_t)&sm_file_inited;
     iov[2].iov_len = sizeof(sm_file_inited);
@@ -228,11 +225,11 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init(ompi_proc_t **procs,
     if (0 == orte_util_compare_name_fields(ORTE_NS_CMP_ALL,
                                            ORTE_PROC_MY_NAME,
                                            &(procs[0]->proc_name))) {
-        /* Check, whether the specified filename is on a network file system */ 
-        if (opal_mmap_on_nfs_warning && opal_path_nfs(file_name)) { 
-            orte_show_help("help-mpi-common-sm.txt", "mmap on nfs", 1, 
-                           orte_process_info.nodename, file_name); 
-        } 
+        /* Check, whether the specified filename is on a network file system */
+        if (opal_path_nfs(file_name)) {
+            orte_show_help("help-mpi-common-sm.txt", "mmap on nfs", 1,
+                           orte_process_info.nodename, file_name);
+        }
         /* process initializing the file */
         fd = open(file_name, O_CREAT|O_RDWR, 0600);
         if (fd < 0) {
@@ -283,7 +280,7 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init(ompi_proc_t **procs,
 
                 /* Free it all -- bad things are going to happen */
                 if (1 == sm_file_inited) {
-                    munmap(map, size);
+                    munmap((void *)map, size);
                     close(fd);
                     unlink(file_name);
                     fd = -1;
@@ -429,7 +426,7 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init(ompi_proc_t **procs,
 
     /* set up the map object */
     map = OBJ_NEW(mca_common_sm_mmap_t);
-    strncpy(map->map_path, file_name, OMPI_PATH_MAX);
+    strncpy(map->map_path, file_name, OPAL_PATH_MAX);
     /* the first entry in the file is the control structure. The first
        entry in the control structure is an mca_common_sm_file_header_t
        element */
@@ -496,9 +493,9 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init(ompi_proc_t **procs,
  * are local, and if they are, calls mca_common_sm_mmap_init().
  */
 mca_common_sm_mmap_t* mca_common_sm_mmap_init_group(ompi_group_t *group,
-                                                    size_t size,
+                                                    size_t size, 
                                                     char *file_name,
-                                                    size_t size_ctl_structure,
+                                                    size_t size_ctl_structure, 
                                                     size_t data_seg_alignment)
 {
     size_t i, group_size;
@@ -512,7 +509,7 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init_group(ompi_group_t *group,
     }
     for (i = 0; i < group_size; ++i) {
         proc = ompi_group_peer_lookup(group,i);
-        if (0 == (proc->proc_flags & OMPI_PROC_FLAG_LOCAL)) {
+        if (!OPAL_PROC_ON_LOCAL_NODE(proc->proc_flags)) {
             free(procs);
             return NULL;
         }
