@@ -26,7 +26,6 @@
 #include <stdio.h>
 #include <ctype.h>
 
-#include "orte/util/show_help.h"
 #include "opal/util/argv.h"
 #include "opal/util/opal_getcwd.h"
 
@@ -35,11 +34,14 @@
 #include "orte/mca/grpcomm/grpcomm.h"
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/rml/rml.h"
+#include "orte/mca/rml/rml_types.h"
+#include "orte/mca/rmaps/rmaps.h"
+#include "orte/mca/rmaps/rmaps_types.h"
+#include "orte/mca/rmaps/base/base.h"
 #include "orte/mca/rml/base/rml_contact.h"
 #include "orte/mca/routed/routed.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
-#include "orte/runtime/orte_data_server.h"
 #include "orte/runtime/orte_wait.h"
 
 #include "ompi/communicator/communicator.h"
@@ -47,7 +49,6 @@
 #include "ompi/proc/proc.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/info/info.h"
-#include "ompi/runtime/ompi_module_exchange.h"
 
 #include "ompi/mca/dpm/base/base.h"
 #include "dpm_orte.h"
@@ -64,9 +65,6 @@ static void recv_cb(int status, orte_process_name_t* sender,
                     opal_buffer_t *buffer,
                     orte_rml_tag_t tag, void *cbdata);
 static void process_cb(int fd, short event, void *data);
-static int parse_port_name(char *port_name,
-                           orte_process_name_t *rproc,
-                           orte_rml_tag_t *tag);
 
 /* API functions */
 static int init(void);
@@ -81,6 +79,9 @@ static int spawn(int count, char **array_of_commands,
                  char *port_name);
 static int dyn_init(void);
 static int open_port(char *port_name, orte_rml_tag_t given_tag);
+static int parse_port_name(char *port_name, char **hnp_uri, char **rml_uri,
+                           orte_rml_tag_t *tag);
+static int route_to_port(char *rml_uri, orte_process_name_t *rproc);
 static int close_port(char *port_name);
 static int finalize(void);
 
@@ -96,6 +97,8 @@ ompi_dpm_base_module_t ompi_dpm_orte_module = {
     ompi_dpm_base_dyn_finalize,
     ompi_dpm_base_mark_dyncomm,
     open_port,
+    parse_port_name,
+    route_to_port, 
     close_port,
     finalize
 };
@@ -148,13 +151,26 @@ static int connect_accept ( ompi_communicator_t *comm, int root,
      * set us up to communicate with it
      */
     if (NULL != port_string && 0 < strlen(port_string)) {
-        /* separate the string into the RML URI and tag - this function performs
-         * whatever route initialization is required by the selected routed module
-         */
-        if (ORTE_SUCCESS != (rc = parse_port_name(port_string, &port, &tag))) {
+        char *hnp_uri, *rml_uri;
+
+        /* separate the string into the HNP and RML URI and tag */
+        if (ORTE_SUCCESS != (rc = parse_port_name(port_string, &hnp_uri, &rml_uri, &tag))) {
             ORTE_ERROR_LOG(rc);
             return rc;
         }
+        /* extract the originating proc's name */
+        if (ORTE_SUCCESS != (rc = orte_rml_base_parse_uris(rml_uri, &port, NULL))) {
+            ORTE_ERROR_LOG(rc);
+            free(hnp_uri); free(rml_uri);
+            return rc;
+        }
+        /* make sure we can route rml messages to the destination job */
+        if (ORTE_SUCCESS != (rc = route_to_port(hnp_uri, &port))) {
+            ORTE_ERROR_LOG(rc);
+            free(hnp_uri); free(rml_uri);
+            return rc;
+        }
+        free(hnp_uri); free(rml_uri);
     }
     
     /* tell the progress engine to tick the event library more
@@ -483,15 +499,17 @@ static int spawn(int count, char **array_of_commands,
 {
     int rc, i, j, counter;
     int have_wdir=0;
-    int valuelen=OMPI_PATH_MAX, flag=0;
-    char cwd[OMPI_PATH_MAX];
-    char host[OMPI_PATH_MAX];  /*** should define OMPI_HOST_MAX ***/
-    char prefix[OMPI_PATH_MAX];
-    char stdin_target[OMPI_PATH_MAX];
+    int flag=0;
+    char cwd[OPAL_PATH_MAX];
+    char host[OPAL_PATH_MAX];  /*** should define OMPI_HOST_MAX ***/
+    char prefix[OPAL_PATH_MAX];
+    char stdin_target[OPAL_PATH_MAX];
+    char params[OPAL_PATH_MAX];
 
     orte_job_t *jdata;
     orte_app_context_t *app;
     bool local_spawn, non_mpi;
+    bool local_bynode = false;
     
     /* parse the info object */
     /* check potentially for:
@@ -545,7 +563,8 @@ static int spawn(int count, char **array_of_commands,
         }
         /* record the number of procs to be generated */
         app->num_procs = array_of_maxprocs[i];
-
+        jdata->num_procs += app->num_procs;
+        
         /* copy over the argv array */
         counter = 1;
 
@@ -581,8 +600,7 @@ static int spawn(int count, char **array_of_commands,
         /* Add environment variable with the contact information for the
            child processes.
         */
-        counter = 1;
-        app->env = (char**)malloc((1+counter) * sizeof(char*));
+        app->env = (char**)malloc(2 * sizeof(char*));
         if (NULL == app->env) {
             ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
             OBJ_RELEASE(jdata);
@@ -602,35 +620,41 @@ static int spawn(int count, char **array_of_commands,
         if ( array_of_info != NULL && array_of_info[i] != MPI_INFO_NULL ) {
 
             /* check for 'host' */
-            ompi_info_get (array_of_info[i], "host", sizeof(host), host, &flag);
+            ompi_info_get (array_of_info[i], "host", sizeof(host) - 1, host, &flag);
             if ( flag ) {
-                opal_argv_append_nosize(&app->dash_host, host);
+                app->dash_host = opal_argv_split(host, ',');
             }
  
             /* check for 'hostfile' */
-            ompi_info_get (array_of_info[i], "hostfile", sizeof(host), host, &flag);
+            ompi_info_get (array_of_info[i], "hostfile", sizeof(host) - 1, host, &flag);
             if ( flag ) {
                 app->hostfile = strdup(host);
             }
             
             /* check for 'add-hostfile' */
-            ompi_info_get (array_of_info[i], "add-hostfile", sizeof(host), host, &flag);
+            ompi_info_get (array_of_info[i], "add-hostfile", sizeof(host) - 1, host, &flag);
             if ( flag ) {
                 app->add_hostfile = strdup(host);
             }
             
-            /* 'path', 'arch', 'file', 'soft', 'add-host'  -- to be implemented */ 
+            /* check for 'add-host' */
+            ompi_info_get (array_of_info[i], "add-host", sizeof(host) - 1, host, &flag);
+            if ( flag ) {
+                app->add_host = opal_argv_split(host, ',');
+            }
+            
+            /* 'path', 'arch', 'file', 'soft'  -- to be implemented */ 
             
             /* check for 'ompi_prefix' (OMPI-specific -- to effect the same
              * behavior as --prefix option to orterun)
              */
-            ompi_info_get (array_of_info[i], "ompi_prefix", sizeof(prefix), prefix, &flag);
+            ompi_info_get (array_of_info[i], "ompi_prefix", sizeof(prefix) - 1, prefix, &flag);
             if ( flag ) {
                 app->prefix_dir = strdup(prefix);
             }
 
             /* check for 'wdir' */ 
-            ompi_info_get (array_of_info[i], "wdir", valuelen, cwd, &flag);
+            ompi_info_get (array_of_info[i], "wdir", sizeof(cwd) - 1, cwd, &flag);
             if ( flag ) {
                 app->cwd = strdup(cwd);
                 have_wdir = 1;
@@ -642,9 +666,65 @@ static int spawn(int count, char **array_of_commands,
              */
             ompi_info_get_bool(array_of_info[i], "ompi_local_slave", &local_spawn, &flag);
             if ( flag && local_spawn ) {
-                jdata->controls |= ORTE_JOB_CONTROL_LOCAL_SPAWN;
+                jdata->controls |= ORTE_JOB_CONTROL_LOCAL_SLAVE;
             }
-         
+
+            /* check for 'map_bynode' */
+            ompi_info_get_bool(array_of_info[i], "map_bynode", &local_bynode, &flag);
+            if ( flag ) {
+                jdata->map = OBJ_NEW(orte_job_map_t);
+                if (NULL == jdata->map) {
+                    ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
+                    return ORTE_ERR_OUT_OF_RESOURCE;
+                }
+                /* load it with the system defaults */
+                jdata->map->policy = orte_default_mapping_policy;
+                jdata->map->npernode = orte_rmaps_base.npernode;
+                jdata->map->nperboard = orte_rmaps_base.nperboard;
+                jdata->map->npersocket = orte_rmaps_base.npersocket;
+                jdata->map->cpus_per_rank = orte_rmaps_base.cpus_per_rank;
+                jdata->map->stride = orte_rmaps_base.stride;
+                jdata->map->oversubscribe = orte_rmaps_base.oversubscribe;
+                jdata->map->display_map = orte_rmaps_base.display_map;
+
+                if( local_bynode ) {
+                    jdata->map->policy = ORTE_MAPPING_BYNODE;
+                }
+                else {
+                    jdata->map->policy = ORTE_MAPPING_BYSLOT;
+                }
+            }
+
+            /* check for 'preload_binary' */
+            ompi_info_get_bool(array_of_info[i], "ompi_preload_binary", &local_spawn, &flag);
+            if ( flag ) {
+                app->preload_binary = true;
+            }
+            
+            /* check for 'preload_libraries' */
+            ompi_info_get_bool(array_of_info[i], "ompi_preload_libraries", &local_spawn, &flag);
+            if ( flag ) {
+                app->preload_libs = true;
+            }
+            
+            /* check for 'preload_files' */ 
+            ompi_info_get (array_of_info[i], "ompi_preload_files", sizeof(cwd) - 1, cwd, &flag);
+            if ( flag ) {
+                app->preload_files = strdup(cwd);
+            }
+            
+            /* check for 'preload_files_dest_dir' */ 
+            ompi_info_get (array_of_info[i], "ompi_preload_files_dest_dir", sizeof(cwd) - 1, cwd, &flag);
+            if ( flag ) {
+                app->preload_files_dest_dir = strdup(cwd);
+            }
+            
+            /* check for 'preload_files_src_dir' */ 
+            ompi_info_get (array_of_info[i], "ompi_preload_files_src_dir", sizeof(cwd) - 1, cwd, &flag);
+            if ( flag ) {
+                app->preload_files_src_dir = strdup(cwd);
+            }
+            
             /* see if this is a non-mpi job - if so, then set the flag so ORTE
              * knows what to do
              */
@@ -653,10 +733,16 @@ static int spawn(int count, char **array_of_commands,
                 jdata->controls |= ORTE_JOB_CONTROL_NON_ORTE_JOB;
             }
             
+            /* see if this is an MCA param that the user wants applied to the child job */
+            ompi_info_get (array_of_info[i], "ompi_param", sizeof(params) - 1, params, &flag);
+            if ( flag ) {
+                opal_argv_append_unique_nosize(&app->env, params, true);
+            }
+            
             /* see if user specified what to do with stdin - defaults to
              * not forwarding stdin to child processes
              */
-            ompi_info_get (array_of_info[i], "ompi_stdin_target", valuelen, stdin_target, &flag);
+            ompi_info_get (array_of_info[i], "ompi_stdin_target", sizeof(stdin_target) - 1, stdin_target, &flag);
             if ( flag ) {
                 if (0 == strcmp(stdin_target, "all")) {
                     jdata->stdin_target = ORTE_VPID_WILDCARD;
@@ -669,15 +755,21 @@ static int spawn(int count, char **array_of_commands,
         }
 
         /* default value: If the user did not tell us where to look for the
-           executable, we assume the current working directory  */
+         * executable, we assume the current working directory, or the preload destination
+         * if it was given
+         */
         if ( !have_wdir ) {
-            if (OMPI_SUCCESS != (rc = opal_getcwd(cwd, OMPI_PATH_MAX))) {
-                ORTE_ERROR_LOG(rc);
-                OBJ_RELEASE(jdata);
-                opal_progress_event_users_decrement();
-                return rc;
+            if (NULL != app->preload_files_dest_dir) {
+                app->cwd = strdup(app->preload_files_dest_dir);
+            } else {
+                if (OMPI_SUCCESS != (rc = opal_getcwd(cwd, OPAL_PATH_MAX))) {
+                    ORTE_ERROR_LOG(rc);
+                    OBJ_RELEASE(jdata);
+                    opal_progress_event_users_decrement();
+                    return rc;
+                }
+                app->cwd = strdup(cwd);
             }
-            app->cwd = strdup(cwd);
         }
         
         /* leave the map info alone - the launcher will
@@ -773,41 +865,43 @@ cleanup:
 }
 
 
-/* HANDLE ACK MESSAGES FROM AN HNP */
-static bool ack_recvd;
-
-static void release_ack(int fd, short event, void *data)
-{
-    orte_message_event_t *mev = (orte_message_event_t*)data;
-    ack_recvd = true;
-    OBJ_RELEASE(mev);
-}
-
-static void recv_ack(int status, orte_process_name_t* sender,
-                     opal_buffer_t* buffer, orte_rml_tag_t tag,
-                     void* cbdata)
-{
-    /* don't process this right away - we need to get out of the recv before
-     * we process the message as it may ask us to do something that involves
-     * more messaging! Instead, setup an event so that the message gets processed
-     * as soon as we leave the recv.
-     *
-     * The macro makes a copy of the buffer, which we release above - the incoming
-     * buffer, however, is NOT released here, although its payload IS transferred
-     * to the message buffer for later processing
+static int route_to_port(char *rml_uri, orte_process_name_t *rproc)
+{    
+    opal_buffer_t route;
+    orte_rml_cmd_flag_t cmd = ORTE_RML_UPDATE_CMD;    
+    int rc;
+    
+    /* We need to ask the routed module to init_routes so it can do the
+     * right thing. In most cases, it will route any messages to the
+     * proc through our HNP - however, this is NOT the case in all
+     * circumstances, so we need to let the routed module decide what
+     * to do.
      */
-    ORTE_MESSAGE_EVENT(sender, buffer, tag, release_ack);    
+    /* pack a cmd so the buffer can be unpacked correctly */
+    OBJ_CONSTRUCT(&route, opal_buffer_t);
+    opal_dss.pack(&route, &cmd, 1, ORTE_RML_CMD);
+    
+    /* pack the provided uri */
+    opal_dss.pack(&route, &rml_uri, 1, OPAL_STRING);
+    
+    /* init the route */
+    if (ORTE_SUCCESS != (rc = orte_routed.init_routes(rproc->jobid, &route))) {
+        ORTE_ERROR_LOG(rc);
+    }
+    OBJ_DESTRUCT(&route);
+    
+    /* nothing more to do here */
+    return rc;
 }
 
-
 static int parse_port_name(char *port_name,
-                           orte_process_name_t *rproc,
-                           orte_rml_tag_t *tag)
+                           char **hnp_uri, 
+                           char **rml_uri,
+                           orte_rml_tag_t *ptag)
 {
-    char *tmpstring=NULL, *ptr, *rml_uri=NULL;
-    orte_rml_cmd_flag_t cmd = ORTE_RML_UPDATE_CMD;
+    char *tmpstring=NULL, *ptr;
+    int tag;
     int rc;
-    opal_buffer_t route;
     
     /* don't mangle the port name */
     tmpstring = strdup(port_name);
@@ -823,7 +917,7 @@ static int parse_port_name(char *port_name,
     ptr++;
     
     /* convert the RML tag */
-    sscanf(ptr,"%d", (int*)tag);
+    sscanf(ptr,"%d", &tag);
     
     /* now split out the second field - the uri of the remote proc */
     if (NULL == (ptr = strchr(tmpstring, '+'))) {
@@ -834,85 +928,18 @@ static int parse_port_name(char *port_name,
     ptr++;
     
     /* save that info */
-    rml_uri = strdup(ptr);
+    if(NULL != hnp_uri) *hnp_uri = tmpstring;
+    else free(tmpstring);
+    if(NULL != rml_uri) *rml_uri = strdup(ptr);
+    if(NULL != ptag) *ptag = tag;
     
-    /* extract the originating proc's name */
-    if (ORTE_SUCCESS != (rc = orte_rml_base_parse_uris(ptr, rproc, NULL))) {
-        ORTE_ERROR_LOG(rc);
-        goto cleanup;
-    }
-    
-    /* if this proc is part of my job family, then I need to
-     * update my RML contact hash table and my routes
-     */
-    if (ORTE_JOB_FAMILY(rproc->jobid) == ORTE_JOB_FAMILY(ORTE_PROC_MY_NAME->jobid)) {
-        OPAL_OUTPUT_VERBOSE((3, ompi_dpm_base_output,
-                             "%s dpm_parse_port: same job family - updating route",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-        
-        /* set the contact info into the hash table */
-        if (ORTE_SUCCESS != (rc = orte_rml.set_contact_info(rml_uri))) {
-            ORTE_ERROR_LOG(rc);
-            goto cleanup;
-        }
-        if (ORTE_SUCCESS != (rc = orte_routed.update_route(rproc, rproc))) {
-            ORTE_ERROR_LOG(rc);
-        }
-        goto cleanup;
-    }
-    
-    /* the proc must be part of another job family. In this case, we
-     * will route any messages to the proc through our HNP. We need
-     * to update the HNP, though, so it knows how to reach the
-     * HNP of the rproc's job family
-     */
-    /* pack a cmd so the buffer can be unpacked correctly */
-    OBJ_CONSTRUCT(&route, opal_buffer_t);
-    opal_dss.pack(&route, &cmd, 1, ORTE_RML_CMD);
-    
-    /* pack the HNP uri */
-    opal_dss.pack(&route, &tmpstring, 1, OPAL_STRING);
-    
-    OPAL_OUTPUT_VERBOSE((3, ompi_dpm_base_output,
-                         "%s dpm_parse_port: %s in diff job family - sending update to %s",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_NAME_PRINT(rproc),
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_HNP)));
-    
-    if (0 > (rc = orte_rml.send_buffer(ORTE_PROC_MY_HNP, &route,
-                                       ORTE_RML_TAG_RML_INFO_UPDATE, 0))) {
-        ORTE_ERROR_LOG(rc);
-        OBJ_DESTRUCT(&route);
-        goto cleanup;
-    }
-    
-    /* wait right here until the HNP acks the update to ensure that
-     * any subsequent messaging can succeed
-     */
-    ack_recvd = false;
-    rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_UPDATE_ROUTE_ACK,
-                                 ORTE_RML_NON_PERSISTENT, recv_ack, NULL);
-    
-    ORTE_PROGRESSED_WAIT(ack_recvd, 0, 1);
-    OBJ_DESTRUCT(&route);
-    
-    OPAL_OUTPUT_VERBOSE((3, ompi_dpm_base_output,
-                         "%s dpm_parse_port: ack recvd",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-    
-    /* our get_route function automatically routes all messages for
-     * other job families via the HNP, so nothing more to do here
-     */
-    rc = ORTE_SUCCESS;
+    return ORTE_SUCCESS;
     
 cleanup:
     /* release the tmp storage */
     if (NULL != tmpstring) {
         free(tmpstring);
     }
-    if (NULL != rml_uri) {
-        free(rml_uri);
-    }
     return rc;
 }
 
