@@ -55,7 +55,7 @@ static void dump_csum_error_data(mca_btl_base_segment_t* segments, size_t num_se
     
     printf("CHECKSUM ERROR DATA\n");
     for (i = 0; i < num_segments; ++i) {
-        printf("Segment %lu", i);
+        printf("Segment %lu", (unsigned long)i);
         data = (uint8_t*)segments[i].seg_addr.pval;
         for (j=0; j < segments[i].seg_len; j++) {
             if (0 == (j % 40)) {
@@ -300,11 +300,11 @@ static int mca_pml_csum_recv_request_ack(
          * registered. 
          */
 
-        if(ompi_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == 0 &&
+        if(opal_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == 0 &&
            hdr->hdr_match.hdr_common.hdr_flags & MCA_PML_CSUM_HDR_FLAGS_CONTIG &&
            rdma_num != 0) {
             unsigned char *base;
-            ompi_convertor_get_current_pointer( &recvreq->req_recv.req_base.req_convertor, (void**)&(base) );
+            opal_convertor_get_current_pointer( &recvreq->req_recv.req_base.req_convertor, (void**)&(base) );
            
             if(hdr->hdr_match.hdr_common.hdr_flags & MCA_PML_CSUM_HDR_FLAGS_PIN)
                 recvreq->req_rdma_cnt = mca_pml_csum_rdma_btls(bml_endpoint,
@@ -327,7 +327,7 @@ static int mca_pml_csum_recv_request_ack(
 
                 /* use converter to figure out the rdma offset for this
                  * request */
-                ompi_convertor_set_position(&recvreq->req_recv.req_base.req_convertor,
+                opal_convertor_set_position(&recvreq->req_recv.req_base.req_convertor,
                         &recvreq->req_send_offset);
 
                 recvreq->req_rdma_cnt =
@@ -496,7 +496,7 @@ void mca_pml_csum_recv_request_progress_frag( mca_pml_csum_recv_request_t* recvr
         if(csum != hdr->hdr_frag.hdr_csum) {
             opal_output(0, "%s:%s:%d: Invalid \'frag data\' - received csum:0x%x  != computed csum:0x%x\n",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), __FILE__, __LINE__, hdr->hdr_frag.hdr_csum, csum);
-            orte_notifier.log(ORTE_NOTIFIER_INFRA,
+            orte_notifier.log(ORTE_NOTIFIER_CRIT, 1,
                               "Checksum data violation: job %s file %s line %d",
                               (NULL == orte_job_ident) ? "UNKNOWN" : orte_job_ident,
                               __FILE__, __LINE__);
@@ -540,7 +540,7 @@ void mca_pml_csum_recv_request_progress_rget( mca_pml_csum_recv_request_t* recvr
      * fall back to copy in/out protocol. It is a pity because buffer on the
      * sender side is already registered. We need to be smarter here, perhaps
      * do couple of RDMA reads */
-    if(ompi_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == true) {
+    if(opal_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == true) {
         mca_pml_csum_recv_request_ack(recvreq, &hdr->hdr_rndv, 0);
         return;
     }
@@ -558,7 +558,7 @@ void mca_pml_csum_recv_request_progress_rget( mca_pml_csum_recv_request_t* recvr
     /* allocate/initialize a fragment */
     for(i = 0; i < hdr->hdr_seg_cnt; i++) {
         frag->rdma_segs[i] = hdr->hdr_segs[i];
-#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         if ((recvreq->req_recv.req_base.req_proc->proc_arch & OPAL_ARCH_ISBIGENDIAN) !=
             (ompi_proc_local()->proc_arch & OPAL_ARCH_ISBIGENDIAN)) {
             size += opal_swap_bytes4(hdr->hdr_segs[i].seg_len);
@@ -642,7 +642,7 @@ void mca_pml_csum_recv_request_progress_rndv( mca_pml_csum_recv_request_t* recvr
         if (csum != hdr->hdr_match.hdr_csum) {
             opal_output(0, "%s:%s:%d: Invalid \'rndv data\' - received csum:0x%x  != computed csum:0x%x\n",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), __FILE__, __LINE__, hdr->hdr_match.hdr_csum, csum);
-            orte_notifier.log(ORTE_NOTIFIER_INFRA,
+            orte_notifier.log(ORTE_NOTIFIER_CRIT, 1,
                               "Checksum data violation: job %s file %s line %d",
                               (NULL == orte_job_ident) ? "UNKNOWN" : orte_job_ident,
                               __FILE__, __LINE__);
@@ -704,7 +704,7 @@ void mca_pml_csum_recv_request_progress_match( mca_pml_csum_recv_request_t* recv
         if (csum != hdr->hdr_match.hdr_csum) {
             opal_output(0, "%s:%s:%d: Invalid \'match data\' - received csum:0x%x  != computed csum:0x%x\n",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), __FILE__, __LINE__, hdr->hdr_match.hdr_csum, csum);
-            orte_notifier.log(ORTE_NOTIFIER_INFRA,
+            orte_notifier.log(ORTE_NOTIFIER_CRIT, 1,
                               "Checksum data violation: job %s file %s line %d",
                               (NULL == orte_job_ident) ? "UNKNOWN" : orte_job_ident,
                               __FILE__, __LINE__);
@@ -841,7 +841,7 @@ int mca_pml_csum_recv_request_schedule_once( mca_pml_csum_recv_request_t* recvre
         /* take lock to protect converter against concurrent access
          * from unpack */
         OPAL_THREAD_LOCK(&recvreq->lock);
-        ompi_convertor_set_position( &recvreq->req_recv.req_base.req_convertor,
+        opal_convertor_set_position( &recvreq->req_recv.req_base.req_convertor,
                                      &recvreq->req_rdma_offset );
 
         /* prepare a descriptor for RDMA */
