@@ -1,15 +1,16 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2007 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2010 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -18,7 +19,6 @@
  */
 
 #include "ompi_config.h"
-#include "opal/sys/cache.h"
 #include "opal/event/event.h"
 #include "mpi.h"
 #include "ompi/runtime/params.h"
@@ -127,6 +127,7 @@ static int mca_pml_ob1_component_open(void)
         opal_output(0, "mca_pml_ob1_component_open: can't find allocator: %s\n", mca_pml_ob1.allocator_name);
         return OMPI_ERROR;
     }
+
     mca_pml_ob1.allocator = allocator_component->allocator_init(true,
                                                                 mca_pml_ob1_seg_alloc,
                                                                 mca_pml_ob1_seg_free, NULL);
@@ -144,8 +145,12 @@ static int mca_pml_ob1_component_close(void)
 {
     int rc;
 
-    if(OMPI_SUCCESS != (rc = mca_bml_base_close()))
-        return rc;
+    if (OMPI_SUCCESS != (rc = mca_bml_base_close())) {
+         return rc;
+    }
+    if (NULL != mca_pml_ob1.allocator_name) {
+        free(mca_pml_ob1.allocator_name);
+    }
 
     return OMPI_SUCCESS;
 }
