@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
@@ -261,11 +261,11 @@ static int mca_pml_ob1_recv_request_ack(
          * registered. 
          */
 
-        if(ompi_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == 0 &&
+        if(opal_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == 0 &&
            hdr->hdr_match.hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_CONTIG &&
            rdma_num != 0) {
             unsigned char *base;
-            ompi_convertor_get_current_pointer( &recvreq->req_recv.req_base.req_convertor, (void**)&(base) );
+            opal_convertor_get_current_pointer( &recvreq->req_recv.req_base.req_convertor, (void**)&(base) );
            
             if(hdr->hdr_match.hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_PIN)
                 recvreq->req_rdma_cnt = mca_pml_ob1_rdma_btls(bml_endpoint,
@@ -288,7 +288,7 @@ static int mca_pml_ob1_recv_request_ack(
 
                 /* use converter to figure out the rdma offset for this
                  * request */
-                ompi_convertor_set_position(&recvreq->req_recv.req_base.req_convertor,
+                opal_convertor_set_position(&recvreq->req_recv.req_base.req_convertor,
                         &recvreq->req_send_offset);
 
                 recvreq->req_rdma_cnt =
@@ -304,11 +304,11 @@ static int mca_pml_ob1_recv_request_ack(
     /* let know to shedule function there is no need to put ACK flag */
     recvreq->req_ack_sent = true;
     return mca_pml_ob1_recv_request_ack_send(proc, hdr->hdr_src_req.lval,
-            recvreq, recvreq->req_send_offset,
-            recvreq->req_send_offset == bytes_received);
+                                             recvreq, recvreq->req_send_offset,
+                                             recvreq->req_send_offset == bytes_received);
 }
 
-                                                                                                            
+
 /**
  * Return resources used by the RDMA
  */
@@ -331,7 +331,7 @@ static void mca_pml_ob1_rget_completion( mca_btl_base_module_t* btl,
 
     mca_pml_ob1_send_fin(recvreq->req_recv.req_base.req_proc,
                          bml_btl,
-                         frag->rdma_hdr.hdr_rget.hdr_des.pval,
+                         frag->rdma_hdr.hdr_rget.hdr_des,
                          des->order, 0); 
 
     /* is receive request complete */
@@ -414,7 +414,7 @@ void mca_pml_ob1_recv_request_progress_frag( mca_pml_ob1_recv_request_t* recvreq
                                              size_t num_segments )
 {
     size_t bytes_received = 0;
-    size_t bytes_delivered = 0;
+    size_t bytes_delivered __opal_attribute_unused__; /* is being set to zero in MCA_PML_OB1_RECV_REQUEST_UNPACK */
     size_t data_offset = 0;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
 
@@ -479,13 +479,12 @@ void mca_pml_ob1_recv_request_progress_rget( mca_pml_ob1_recv_request_t* recvreq
     recvreq->req_recv.req_bytes_packed = hdr->hdr_rndv.hdr_msg_length;
 
     MCA_PML_OB1_RECV_REQUEST_MATCHED(recvreq, &hdr->hdr_rndv.hdr_match);
-
     
     /* if receive buffer is not contiguous we can't just RDMA read into it, so
      * fall back to copy in/out protocol. It is a pity because buffer on the
      * sender side is already registered. We need to be smarter here, perhaps
      * do couple of RDMA reads */
-    if(ompi_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == true) {
+    if(opal_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == true) {
         mca_pml_ob1_recv_request_ack(recvreq, &hdr->hdr_rndv, 0);
         return;
     }
@@ -503,7 +502,7 @@ void mca_pml_ob1_recv_request_progress_rget( mca_pml_ob1_recv_request_t* recvreq
     /* allocate/initialize a fragment */
     for(i = 0; i < hdr->hdr_seg_cnt; i++) {
         frag->rdma_segs[i] = hdr->hdr_segs[i];
-#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         if ((recvreq->req_recv.req_base.req_proc->proc_arch & OPAL_ARCH_ISBIGENDIAN) !=
             (ompi_proc_local()->proc_arch & OPAL_ARCH_ISBIGENDIAN)) {
             size += opal_swap_bytes4(hdr->hdr_segs[i].seg_len);
@@ -540,7 +539,7 @@ void mca_pml_ob1_recv_request_progress_rndv( mca_pml_ob1_recv_request_t* recvreq
                                              size_t num_segments )
 {
     size_t bytes_received = 0;
-    size_t bytes_delivered = 0;
+    size_t bytes_delivered __opal_attribute_unused__; /* is being set to zero in MCA_PML_OB1_RECV_REQUEST_UNPACK */
     size_t data_offset = 0;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
 
@@ -597,7 +596,8 @@ void mca_pml_ob1_recv_request_progress_match( mca_pml_ob1_recv_request_t* recvre
                                               mca_btl_base_segment_t* segments,
                                               size_t num_segments )
 {
-    size_t bytes_received = 0, bytes_delivered = 0, data_offset = 0;
+    size_t bytes_received = 0, data_offset = 0;
+    size_t bytes_delivered __opal_attribute_unused__; /* is being set to zero in MCA_PML_OB1_RECV_REQUEST_UNPACK */
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
 
     MCA_PML_OB1_COMPUTE_SEGMENT_LENGTH( segments, num_segments,
@@ -672,7 +672,7 @@ void mca_pml_ob1_recv_request_matched_probe( mca_pml_ob1_recv_request_t* recvreq
     recvreq->req_recv.req_base.req_ompi.req_status.MPI_TAG = hdr->hdr_match.hdr_tag;
     recvreq->req_recv.req_base.req_ompi.req_status.MPI_SOURCE = hdr->hdr_match.hdr_src;
     recvreq->req_bytes_received = bytes_packed;
-    recvreq->req_bytes_delivered = bytes_packed;
+    recvreq->req_bytes_expected = bytes_packed;
     recv_request_pml_complete(recvreq);
 }
 
@@ -750,7 +750,7 @@ int mca_pml_ob1_recv_request_schedule_once( mca_pml_ob1_recv_request_t* recvreq,
         /* take lock to protect converter against concurrent access
          * from unpack */
         OPAL_THREAD_LOCK(&recvreq->lock);
-        ompi_convertor_set_position( &recvreq->req_recv.req_base.req_convertor,
+        opal_convertor_set_position( &recvreq->req_recv.req_base.req_convertor,
                                      &recvreq->req_rdma_offset );
 
         /* prepare a descriptor for RDMA */
@@ -930,9 +930,9 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
 
     /* init/re-init the request */
     req->req_lock = 0;
-    req->req_pipeline_depth  = 0;
-    req->req_bytes_received  = 0;
-    req->req_bytes_delivered = 0;
+    req->req_pipeline_depth = 0;
+    req->req_bytes_received = 0;
+    req->req_bytes_expected = 0;
     /* What about req_rdma_cnt ? */
     req->req_rdma_idx = 0;
     req->req_pending = false;
@@ -946,7 +946,7 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
      * the cost of the request lock.
      */
     PERUSE_TRACE_COMM_EVENT(PERUSE_COMM_SEARCH_UNEX_Q_BEGIN,
-                             &(req->req_recv.req_base), PERUSE_RECV);
+                            &(req->req_recv.req_base), PERUSE_RECV);
 
     /* assign sequence number */
     req->req_recv.req_base.req_sequence = comm->recv_sequence++;
@@ -955,7 +955,7 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
     if(req->req_recv.req_base.req_peer == OMPI_ANY_SOURCE) {
         frag = recv_req_match_wild(req, &proc);
         queue = &comm->wild_receives;
-#if !OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#if !OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         /* As we are in a homogeneous environment we know that all remote
          * architectures are exactly the same as the local one. Therefore,
          * we can safely construct the convertor based on the proc
@@ -965,7 +965,7 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
             req->req_recv.req_base.req_proc = ompi_proc_local_proc;
             prepare_recv_req_converter(req);
         }
-#endif  /* !OMPI_ENABLE_HETEROGENEOUS_SUPPORT */
+#endif  /* !OPAL_ENABLE_HETEROGENEOUS_SUPPORT */
     } else {
         proc = &comm->procs[req->req_recv.req_base.req_peer];
         req->req_recv.req_base.req_proc = proc->ompi_proc;
@@ -977,7 +977,7 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
 
     if(OPAL_UNLIKELY(NULL == frag)) {
         PERUSE_TRACE_COMM_EVENT(PERUSE_COMM_SEARCH_UNEX_Q_END,
-                &(req->req_recv.req_base), PERUSE_RECV);
+                                &(req->req_recv.req_base), PERUSE_RECV);
         /* We didn't find any matches.  Record this irecv so we can match
            it when the message comes in. */
         append_recv_req_to_queue(queue, req);
@@ -986,22 +986,22 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
     } else {
         if(OPAL_LIKELY(!IS_PROB_REQ(req))) {
             PERUSE_TRACE_COMM_EVENT(PERUSE_COMM_REQ_MATCH_UNEX,
-                    &(req->req_recv.req_base), PERUSE_RECV);
+                                    &(req->req_recv.req_base), PERUSE_RECV);
 
+            hdr = (mca_pml_ob1_hdr_t*)frag->segments->seg_addr.pval;
             PERUSE_TRACE_MSG_EVENT(PERUSE_COMM_MSG_REMOVE_FROM_UNEX_Q,
-                    req->req_recv.req_base.req_comm,
-                    req->req_recv.req_base.req_ompi.req_status.MPI_SOURCE,
-                    req->req_recv.req_base.req_ompi.req_status.MPI_TAG,
-                    PERUSE_RECV);
+                                   req->req_recv.req_base.req_comm,
+                                   hdr->hdr_match.hdr_src,
+                                   hdr->hdr_match.hdr_tag,
+                                   PERUSE_RECV);
 
             PERUSE_TRACE_COMM_EVENT(PERUSE_COMM_SEARCH_UNEX_Q_END,
-                    &(req->req_recv.req_base), PERUSE_RECV);
+                                    &(req->req_recv.req_base), PERUSE_RECV);
 
             opal_list_remove_item(&proc->unexpected_frags,
                                   (opal_list_item_t*)frag);
             OPAL_THREAD_UNLOCK(&comm->matching_lock);
             
-            hdr = (mca_pml_ob1_hdr_t*)frag->segments->seg_addr.pval;
             switch(hdr->hdr_common.hdr_type) {
             case MCA_PML_OB1_HDR_TYPE_MATCH:
                 mca_pml_ob1_recv_request_progress_match(req, frag->btl, frag->segments,
