@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
+ * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,10 +21,13 @@
 #include <stdio.h>
 
 #include "ompi/mpi/c/bindings.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/runtime/params.h"
+#include "ompi/communicator/communicator.h"
+#include "ompi/errhandler/errhandler.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Alltoallw = PMPI_Alltoallw
 #endif
 
@@ -53,8 +56,8 @@ int MPI_Alltoallw(void *sendbuf, int *sendcounts, int *sdispls,
             memchecker_datatype(sendtypes[i]);
             memchecker_datatype(recvtypes[i]);
             
-            ompi_ddt_type_extent(sendtypes[i], &send_ext);
-            ompi_ddt_type_extent(recvtypes[i], &recv_ext);
+            ompi_datatype_type_extent(sendtypes[i], &send_ext);
+            ompi_datatype_type_extent(recvtypes[i], &recv_ext);
 
             memchecker_call(&opal_memchecker_base_isdefined,
                             (char *)(sendbuf)+sdispls[i]*send_ext,
