@@ -22,10 +22,13 @@
 #include <stdio.h>
 
 #include "ompi/mpi/c/bindings.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/runtime/params.h"
+#include "ompi/communicator/communicator.h"
+#include "ompi/errhandler/errhandler.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Gather = PMPI_Gather
 #endif
 
@@ -47,7 +50,7 @@ int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
         ptrdiff_t ext;
 
         rank = ompi_comm_rank(comm);
-        ompi_ddt_type_extent(recvtype, &ext);
+        ompi_datatype_type_extent(recvtype, &ext);
 
         memchecker_comm(comm);
         if(OMPI_COMM_IS_INTRA(comm)) {
