@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -19,10 +20,12 @@
 #include "ompi_config.h"
 
 #include "ompi/mpi/c/bindings.h"
+#include "ompi/runtime/params.h"
+#include "ompi/errhandler/errhandler.h"
 #include "ompi/mpi/f77/fint_2_int.h"
 #include "ompi/op/op.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Op_f2c = PMPI_Op_f2c
 #endif
 
@@ -37,6 +40,8 @@ MPI_Op MPI_Op_f2c(MPI_Fint op_f)
 {
     int op_index = OMPI_FINT_2_INT(op_f);
 
+    OPAL_CR_NOOP_PROGRESS();
+
     /* Error checking */
 
     if (MPI_PARAM_CHECK) {
@@ -49,9 +54,9 @@ MPI_Op MPI_Op_f2c(MPI_Fint op_f)
     
     if (op_index < 0 || 
         op_index >= 
-        ompi_pointer_array_get_size(ompi_op_f_to_c_table)) {
+        opal_pointer_array_get_size(ompi_op_f_to_c_table)) {
         return NULL;
     }
 
-    return ompi_pointer_array_get_item(ompi_op_f_to_c_table, op_index);
+    return (MPI_Op)opal_pointer_array_get_item(ompi_op_f_to_c_table, op_index);
 }
