@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -19,10 +20,14 @@
 #include <stdio.h>
 
 #include "ompi/mpi/c/bindings.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/runtime/params.h"
+#include "ompi/communicator/communicator.h"
+#include "ompi/errhandler/errhandler.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/op/op.h"
+#include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Reduce_scatter = PMPI_Reduce_scatter
 #endif
 
@@ -38,6 +43,33 @@ int MPI_Reduce_scatter(void *sendbuf, void *recvbuf, int *recvcounts,
 {
     int i, err, size, count;
 
+    MEMCHECKER(
+        int rank;
+        
+        size = ompi_comm_size(comm);
+        rank = ompi_comm_rank(comm);
+        for (count = i = 0; i < size; ++i) {
+            if (0 == recvcounts[i]) {
+                ++count;
+            }
+        }
+
+        memchecker_comm(comm);
+        memchecker_datatype(datatype);
+        
+        /* check receive buffer of current proccess, whether it's addressable. */
+        memchecker_call(&opal_memchecker_base_isaddressable, recvbuf,
+                        recvcounts[rank], datatype);
+        
+        /* check whether the actual send buffer is defined. */
+        if(MPI_IN_PLACE == sendbuf) {
+            memchecker_call(&opal_memchecker_base_isdefined, recvbuf, count, datatype);
+        } else {
+            memchecker_call(&opal_memchecker_base_isdefined, sendbuf, count, datatype);
+          
+        }
+    );
+
     if (MPI_PARAM_CHECK) {
         char *msg;
         err = MPI_SUCCESS;
@@ -50,7 +82,7 @@ int MPI_Reduce_scatter(void *sendbuf, void *recvbuf, int *recvcounts,
         /* Unrooted operation; same checks for all ranks on both
            intracommunicators and intercommunicators */
 
-        else if (MPI_OP_NULL == op) {
+        else if (MPI_OP_NULL == op || NULL == op) {
           err = MPI_ERR_OP;
         } else if (!ompi_op_is_valid(op, datatype, &msg, FUNC_NAME)) {
             int ret = OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_OP, msg);
@@ -88,12 +120,15 @@ int MPI_Reduce_scatter(void *sendbuf, void *recvbuf, int *recvcounts,
     if (size == count) {
         return MPI_SUCCESS;
     }
-        
+
+    OPAL_CR_ENTER_LIBRARY();
+
     /* Invoke the coll component to perform the back-end operation */
 
     OBJ_RETAIN(op);
     err = comm->c_coll.coll_reduce_scatter(sendbuf, recvbuf, recvcounts,
-                                           datatype, op, comm);
+                                           datatype, op, comm,
+                                           comm->c_coll.coll_reduce_scatter_module);
     OBJ_RELEASE(op);
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
