@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -19,9 +20,13 @@
 #include <stdio.h>
 
 #include "ompi/mpi/c/bindings.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/runtime/params.h"
+#include "ompi/communicator/communicator.h"
+#include "ompi/errhandler/errhandler.h"
+#include "ompi/datatype/ompi_datatype.h"
+#include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Scatterv = PMPI_Scatterv
 #endif
 
@@ -37,7 +42,49 @@ int MPI_Scatterv(void *sendbuf, int *sendcounts, int *displs,
                  MPI_Datatype recvtype, int root, MPI_Comm comm) 
 {
     int i, size, err;
-    
+
+    MEMCHECKER(
+        ptrdiff_t ext;
+
+        size = ompi_comm_size(comm);
+        ompi_datatype_type_extent(recvtype, &ext);
+
+        memchecker_comm(comm);
+        if(OMPI_COMM_IS_INTRA(comm)) {
+              if(ompi_comm_rank(comm) == root) {
+                memchecker_datatype(sendtype);
+                /* check whether root's send buffer is defined. */
+                for (i = 0; i < size; i++) {
+                    memchecker_call(&opal_memchecker_base_isdefined,
+                                    (char *)(sendbuf)+displs[i]*ext,
+                                    sendcounts[i], sendtype);
+                }
+                if(MPI_IN_PLACE != recvbuf) {
+                    memchecker_datatype(recvtype);
+                    /* check whether receive buffer is addressable. */
+                    memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, recvcount, recvtype);
+                }
+              } else {
+                  memchecker_datatype(recvtype);
+                  /* check whether receive buffer is addressable. */
+                  memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, recvcount, recvtype);
+              }
+        } else {
+            if(MPI_ROOT == root) {
+                  memchecker_datatype(sendtype);
+                  /* check whether root's send buffer is defined. */
+                  for (i = 0; i < size; i++) {
+                      memchecker_call(&opal_memchecker_base_isdefined,
+                                      (char *)(sendbuf)+displs[i]*ext,
+                                      sendcounts[i], sendtype);
+                  }
+            } else if (MPI_PROC_NULL != root) {
+                /* check whether receive buffer is addressable. */
+                memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, recvcount, recvtype);
+            }
+        }
+    );
+
     if (MPI_PARAM_CHECK) {
         err = MPI_SUCCESS;
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
@@ -45,7 +92,7 @@ int MPI_Scatterv(void *sendbuf, int *sendcounts, int *displs,
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM, 
                                           FUNC_NAME);
         } else if ((ompi_comm_rank(comm) != root && MPI_IN_PLACE == recvbuf) ||
-                   MPI_IN_PLACE == sendbuf) {
+                   (ompi_comm_rank(comm) == root && MPI_IN_PLACE == sendbuf)) {
             return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_ARG, FUNC_NAME);
         }
 
@@ -65,7 +112,7 @@ int MPI_Scatterv(void *sendbuf, int *sendcounts, int *displs,
                                                 FUNC_NAME);
               }
               
-              if (MPI_DATATYPE_NULL == recvtype) {
+              if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
                   return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_TYPE, 
                                                 FUNC_NAME); 
               }
@@ -109,7 +156,7 @@ int MPI_Scatterv(void *sendbuf, int *sendcounts, int *displs,
               return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_COUNT, FUNC_NAME);
             }
 
-            if (MPI_DATATYPE_NULL == recvtype) {
+            if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
               return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_TYPE, FUNC_NAME); 
             }
           }
@@ -136,15 +183,12 @@ int MPI_Scatterv(void *sendbuf, int *sendcounts, int *displs,
         }
     }
 
-    /* If we have nothing to do, just return */
-
-    if (0 == recvcount && ompi_comm_rank(comm) != root) {
-        return MPI_SUCCESS;
-    }
+    OPAL_CR_ENTER_LIBRARY();
 
     /* Invoke the coll component to perform the back-end operation */
 	
     err = comm->c_coll.coll_scatterv(sendbuf, sendcounts, displs, sendtype, 
-                                     recvbuf, recvcount, recvtype, root, comm);
+                                     recvbuf, recvcount, recvtype, root, comm,
+                                    comm->c_coll.coll_scatterv_module);
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
