@@ -20,7 +20,7 @@
 
 #include "ompi/mpi/f77/bindings.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILE_LAYER
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILE_LAYER
 #pragma weak PMPI_FILE_SEEK_SHARED = mpi_file_seek_shared_f
 #pragma weak pmpi_file_seek_shared = mpi_file_seek_shared_f
 #pragma weak pmpi_file_seek_shared_ = mpi_file_seek_shared_f
@@ -35,14 +35,14 @@ OMPI_GENERATE_F77_BINDINGS (PMPI_FILE_SEEK_SHARED,
                            (fh, offset, whence, ierr) )
 #endif
 
-#if OMPI_HAVE_WEAK_SYMBOLS
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_FILE_SEEK_SHARED = mpi_file_seek_shared_f
 #pragma weak mpi_file_seek_shared = mpi_file_seek_shared_f
 #pragma weak mpi_file_seek_shared_ = mpi_file_seek_shared_f
 #pragma weak mpi_file_seek_shared__ = mpi_file_seek_shared_f
 #endif
 
-#if ! OMPI_HAVE_WEAK_SYMBOLS && ! OMPI_PROFILE_LAYER
+#if ! OPAL_HAVE_WEAK_SYMBOLS && ! OMPI_PROFILE_LAYER
 OMPI_GENERATE_F77_BINDINGS (MPI_FILE_SEEK_SHARED,
                            mpi_file_seek_shared,
                            mpi_file_seek_shared_,
@@ -53,7 +53,7 @@ OMPI_GENERATE_F77_BINDINGS (MPI_FILE_SEEK_SHARED,
 #endif
 
 
-#if OMPI_PROFILE_LAYER && ! OMPI_HAVE_WEAK_SYMBOLS
+#if OMPI_PROFILE_LAYER && ! OPAL_HAVE_WEAK_SYMBOLS
 #include "ompi/mpi/f77/profile/defines.h"
 #endif
 
@@ -62,6 +62,6 @@ void mpi_file_seek_shared_f(MPI_Fint *fh, MPI_Offset *offset,
 {    
     MPI_File c_fh = MPI_File_f2c(*fh);
 
-    *ierr = OMPI_INT_2_FINT(MPI_File_seek_shared(c_fh, (MPI_Offset) *offset,
+    *ierr = OMPI_INT_2_FINT(MPI_File_seek(c_fh, (MPI_Offset) *offset,
 					  OMPI_FINT_2_INT(*whence)));
 }
