@@ -20,9 +20,9 @@
 
 #include "ompi/mpi/f77/bindings.h"
 #include "ompi/attribute/attribute.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILE_LAYER
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILE_LAYER
 #pragma weak PMPI_TYPE_SET_ATTR = mpi_type_set_attr_f
 #pragma weak pmpi_type_set_attr = mpi_type_set_attr_f
 #pragma weak pmpi_type_set_attr_ = mpi_type_set_attr_f
@@ -37,14 +37,14 @@ OMPI_GENERATE_F77_BINDINGS (PMPI_TYPE_SET_ATTR,
                            (type, type_keyval, attr_val, ierr) )
 #endif
 
-#if OMPI_HAVE_WEAK_SYMBOLS
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_TYPE_SET_ATTR = mpi_type_set_attr_f
 #pragma weak mpi_type_set_attr = mpi_type_set_attr_f
 #pragma weak mpi_type_set_attr_ = mpi_type_set_attr_f
 #pragma weak mpi_type_set_attr__ = mpi_type_set_attr_f
 #endif
 
-#if ! OMPI_HAVE_WEAK_SYMBOLS && ! OMPI_PROFILE_LAYER
+#if ! OPAL_HAVE_WEAK_SYMBOLS && ! OMPI_PROFILE_LAYER
 OMPI_GENERATE_F77_BINDINGS (MPI_TYPE_SET_ATTR,
                            mpi_type_set_attr,
                            mpi_type_set_attr_,
@@ -55,7 +55,7 @@ OMPI_GENERATE_F77_BINDINGS (MPI_TYPE_SET_ATTR,
 #endif
 
 
-#if OMPI_PROFILE_LAYER && ! OMPI_HAVE_WEAK_SYMBOLS
+#if OMPI_PROFILE_LAYER && ! OPAL_HAVE_WEAK_SYMBOLS
 #include "ompi/mpi/f77/profile/defines.h"
 #endif
 
