@@ -5,10 +5,12 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2010 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -47,6 +49,16 @@
 #define OMPI_BUILDING 1
 #endif
 
+/*
+ * Flex is trying to include the unistd.h file. As there is no configure
+ * option or this, the flex generated files will try to include the file
+ * even on platforms without unistd.h (such as Windows). Therefore, if we
+ * know this file is not available, we can prevent flex from including it.
+ */
+#ifndef HAVE_UNISTD_H
+#define YY_NO_UNISTD_H
+#endif
+
 /***********************************************************************
  *
  * code that should be in ompi_config_bottom.h regardless of build
@@ -55,9 +67,9 @@
  **********************************************************************/
 
 /* Do we have posix or solaris thread lib */
-#define OMPI_HAVE_THREADS (OMPI_HAVE_POSIX_THREADS || OMPI_HAVE_SOLARIS_THREADS)
+#define OPAL_HAVE_THREADS (OPAL_HAVE_POSIX_THREADS || OPAL_HAVE_SOLARIS_THREADS)
 /* Do we have thread support? */
-#define OMPI_HAVE_THREAD_SUPPORT (OMPI_ENABLE_MPI_THREADS || OMPI_ENABLE_PROGRESS_THREADS)
+#define OPAL_HAVE_THREAD_SUPPORT (OPAL_ENABLE_MULTI_THREADS || OPAL_ENABLE_PROGRESS_THREADS)
 
 /*
  * BEGIN_C_DECLS should be used at the beginning of your declarations,
@@ -78,7 +90,7 @@
  * The attribute definition should be included before any potential
  * usage.
  */
-#if OMPI_HAVE_ATTRIBUTE_ALIGNED
+#if OPAL_HAVE_ATTRIBUTE_ALIGNED
 #    define __opal_attribute_aligned__(a)    __attribute__((__aligned__(a)))
 #    define __opal_attribute_aligned_max__   __attribute__((__aligned__))
 #else
@@ -86,61 +98,68 @@
 #    define __opal_attribute_aligned_max__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_ALWAYS_INLINE
+#if OPAL_HAVE_ATTRIBUTE_ALWAYS_INLINE
 #    define __opal_attribute_always_inline__ __attribute__((__always_inline__))
 #else
 #    define __opal_attribute_always_inline__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_COLD
+#if OPAL_HAVE_ATTRIBUTE_COLD
 #    define __opal_attribute_cold__          __attribute__((__cold__))
 #else
 #    define __opal_attribute_cold__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_CONST
+#if OPAL_HAVE_ATTRIBUTE_CONST
 #    define __opal_attribute_const__         __attribute__((__const__))
 #else
 #    define __opal_attribute_const__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_DEPRECATED
+#if OPAL_HAVE_ATTRIBUTE_DEPRECATED
 #    define __opal_attribute_deprecated__    __attribute__((__deprecated__))
 #else
 #    define __opal_attribute_deprecated__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_FORMAT
+#if OPAL_HAVE_ATTRIBUTE_FORMAT
 #    define __opal_attribute_format__(a,b,c) __attribute__((__format__(a, b, c)))
 #else
 #    define __opal_attribute_format__(a,b,c)
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_HOT
+/* Use this __atribute__ on function-ptr declarations, only */
+#if OPAL_HAVE_ATTRIBUTE_FORMAT_FUNCPTR
+#    define __opal_attribute_format_funcptr__(a,b,c) __attribute__((__format__(a, b, c)))
+#else
+#    define __opal_attribute_format_funcptr__(a,b,c)
+#endif
+
+#if OPAL_HAVE_ATTRIBUTE_HOT
 #    define __opal_attribute_hot__           __attribute__((__hot__))
 #else
 #    define __opal_attribute_hot__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_MALLOC
+#if OPAL_HAVE_ATTRIBUTE_MALLOC
 #    define __opal_attribute_malloc__        __attribute__((__malloc__))
 #else
 #    define __opal_attribute_malloc__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_MAY_ALIAS
+#if OPAL_HAVE_ATTRIBUTE_MAY_ALIAS
 #    define __opal_attribute_may_alias__     __attribute__((__may_alias__))
 #else
 #    define __opal_attribute_may_alias__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_NO_INSTRUMENT_FUNCTION
+#if OPAL_HAVE_ATTRIBUTE_NO_INSTRUMENT_FUNCTION
 #    define __opal_attribute_no_instrument_function__  __attribute__((__no_instrument_function__))
 #else
 #    define __opal_attribute_no_instrument_function__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_NONNULL
+#if OPAL_HAVE_ATTRIBUTE_NONNULL
 #    define __opal_attribute_nonnull__(a)    __attribute__((__nonnull__(a)))
 #    define __opal_attribute_nonnull_all__   __attribute__((__nonnull__))
 #else
@@ -148,49 +167,56 @@
 #    define __opal_attribute_nonnull_all__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_NORETURN
+#if OPAL_HAVE_ATTRIBUTE_NORETURN
 #    define __opal_attribute_noreturn__      __attribute__((__noreturn__))
 #else
 #    define __opal_attribute_noreturn__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_PACKED
+/* Use this __atribute__ on function-ptr declarations, only */
+#if OPAL_HAVE_ATTRIBUTE_NORETURN_FUNCPTR
+#    define __opal_attribute_noreturn_funcptr__  __attribute__((__noreturn__))
+#else
+#    define __opal_attribute_noreturn_funcptr__
+#endif
+
+#if OPAL_HAVE_ATTRIBUTE_PACKED
 #    define __opal_attribute_packed__        __attribute__((__packed__))
 #else
 #    define __opal_attribute_packed__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_PURE
+#if OPAL_HAVE_ATTRIBUTE_PURE
 #    define __opal_attribute_pure__          __attribute__((__pure__))
 #else
 #    define __opal_attribute_pure__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_SENTINEL
+#if OPAL_HAVE_ATTRIBUTE_SENTINEL
 #    define __opal_attribute_sentinel__      __attribute__((__sentinel__))
 #else
 #    define __opal_attribute_sentinel__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_UNUSED
+#if OPAL_HAVE_ATTRIBUTE_UNUSED
 #    define __opal_attribute_unused__        __attribute__((__unused__))
 #else
 #    define __opal_attribute_unused__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_VISIBILITY
+#if OPAL_HAVE_ATTRIBUTE_VISIBILITY
 #    define __opal_attribute_visibility__(a) __attribute__((__visibility__(a)))
 #else
 #    define __opal_attribute_visibility__(a)
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_WARN_UNUSED_RESULT
+#if OPAL_HAVE_ATTRIBUTE_WARN_UNUSED_RESULT
 #    define __opal_attribute_warn_unused_result__ __attribute__((__warn_unused_result__))
 #else
 #    define __opal_attribute_warn_unused_result__
 #endif
 
-#if OMPI_HAVE_ATTRIBUTE_WEAK_ALIAS
+#if OPAL_HAVE_ATTRIBUTE_WEAK_ALIAS
 #    define __opal_attribute_weak_alias__(a) __attribute__((__weak__, __alias__(a)))
 #else
 #    define __opal_attribute_weak_alias__(a)
@@ -231,7 +257,7 @@
 #  endif  /* defined(_USRDLL) */
 #  include "opal/win32/win_compat.h"
 #else
-#  if OMPI_C_HAVE_VISIBILITY
+#  if OPAL_C_HAVE_VISIBILITY
 #    define OPAL_DECLSPEC           __opal_attribute_visibility__("default")
 #    define OPAL_MODULE_DECLSPEC    __opal_attribute_visibility__("default")
 #  else
@@ -244,12 +270,13 @@
  * Do we have <stdint.h>?
  */
 #ifdef HAVE_STDINT_H
-#if defined(__cplusplus) && !defined(__STDC_LIMIT_MACROS)
+#if !defined(__STDC_LIMIT_MACROS) && (defined(c_plusplus) || defined (__cplusplus))
 /* When using a C++ compiler, the max / min value #defines for std
    types are only included if __STDC_LIMIT_MACROS is set before
    including stdint.h */
 #define __STDC_LIMIT_MACROS
 #endif
+#include "opal_config.h"
 #include <stdint.h>
 #else
 #include "opal_stdint.h"
@@ -265,42 +292,62 @@
 #if OMPI_BUILDING
 
 #ifndef HAVE_PTRDIFF_T
-typedef OMPI_PTRDIFF_TYPE ptrdiff_t;
+typedef OPAL_PTRDIFF_TYPE ptrdiff_t;
 #endif
 
 /*
  * If we're in C, we may need to bring in the bool type and true/false
- * constants.  OMPI_NEED_C_BOOL will be true if the compiler either
+ * constants.  OPAL_NEED_C_BOOL will be true if the compiler either
  * needs <stdbool.h> or doesn't define the bool type at all.
  */
 #if !(defined(c_plusplus) || defined(__cplusplus))
-#    if OMPI_NEED_C_BOOL
-#        if OMPI_USE_STDBOOL_H
+#    if OPAL_NEED_C_BOOL
+#        if OPAL_USE_STDBOOL_H
              /* If we're using <stdbool.h>, there is an implicit
                 assumption that the C++ bool is the same size and has
-                the same alignment. */
+                the same alignment.  However, configure may have
+                disabled the MPI C++ bindings, so if "_Bool" exists,
+                then use that sizeof. */
 #            include <stdbool.h>
+             /* This section exists because AC_SIZEOF(bool) may not be
+                run in configure if we're not building the MPI C++
+                bindings. */
+#            undef SIZEOF_BOOL
+#            if SIZEOF__BOOL > 0
+#                define SIZEOF_BOOL SIZEOF__BOOL
+#            else
+                 /* If all else fails, assume it's 1 */
+#                define SIZEOF_BOOL 1
+#            endif
 #        else
              /* We need to create a bool type and ensure that it's the
                 same size / alignment as the C++ bool size /
                 alignment */
 #            define false 0
 #            define true 1
-#            if SIZEOF_BOOL == SIZEOF_CHAR && OMPI_ALIGNMENT_CXX_BOOL == OMPI_ALIGNMENT_CHAR
+#            if SIZEOF_BOOL == SIZEOF_CHAR && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_CHAR
 typedef unsigned char bool;
-#            elif SIZEOF_BOOL == SIZEOF_SHORT && OMPI_ALIGNMENT_CXX_BOOL == OMPI_ALIGNMENT_SHORT
+#            elif SIZEOF_BOOL == SIZEOF_SHORT && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_SHORT
 typedef short bool;
-#            elif SIZEOF_BOOL == SIZEOF_INT && OMPI_ALIGNMENT_CXX_BOOL == OMPI_ALIGNMENT_INT
+#            elif SIZEOF_BOOL == SIZEOF_INT && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_INT
 typedef int bool;
-#            elif SIZEOF_BOOL == SIZEOF_LONG && OMPI_ALIGNMENT_CXX_BOOL == OMPI_ALIGNMENT_LONG
+#            elif SIZEOF_BOOL == SIZEOF_LONG && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_LONG
 typedef long bool;
-#            elif defined(SIZEOF_LONG_LONG) && defined(OMPI_ALIGNMENT_LONG) && SIZEOF_BOOL == SIZEOF_LONG && OMPI_ALIGNMENT_CXX_BOOL == OMPI_ALIGNMENT_LONG
+#            elif defined(SIZEOF_LONG_LONG) && defined(OPAL_ALIGNMENT_LONG) && SIZEOF_BOOL == SIZEOF_LONG && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_LONG
 typedef long long bool;
+             /* If we have _Bool, use that */
+#            elif SIZEOF__BOOL > 0
+#                undef SIZEOF_BOOL
+#                define bool _Bool
+#                define SIZEOF_BOOL SIZEOF__BOOL
 #            else
-#                error Cannot find a C type that corresponds to the size and alignment of C++ bool!
+             /* If all else fails, just make bool be an unsigned char
+                and size of 1 */
+typedef unsigned char bool;
+#                define SIZEOF_BOOL 1
 #            endif
-#        endif  /* OMPI_USE_STDBOOL_H */
-#    endif  /* OMPI_NEED_C_BOOL */
+#        endif  /* OPAL_USE_STDBOOL_H */
+#    endif  /* OPAL_NEED_C_BOOL */
 #endif
 
 /*
@@ -311,11 +358,11 @@ typedef long long bool;
 #include <sys/param.h>
 #endif
 #if defined(PATH_MAX)
-#define OMPI_PATH_MAX	(PATH_MAX + 1)
+#define OPAL_PATH_MAX	(PATH_MAX + 1)
 #elif defined(_POSIX_PATH_MAX)
-#define OMPI_PATH_MAX	(_POSIX_PATH_MAX + 1)
+#define OPAL_PATH_MAX	(_POSIX_PATH_MAX + 1)
 #else
-#define OMPI_PATH_MAX	256
+#define OPAL_PATH_MAX	256
 #endif
 
 /*
@@ -349,7 +396,7 @@ typedef long long bool;
  * this stuff enabled (like the memory manager code) a way to turn us
  * off
  */
-#if OMPI_ENABLE_MEM_DEBUG && !defined(OMPI_DISABLE_ENABLE_MEM_DEBUG)
+#if OPAL_ENABLE_MEM_DEBUG && !defined(OPAL_DISABLE_ENABLE_MEM_DEBUG)
 
 /* It is safe to include opal/util/malloc.h here because a) it will only
    happen when we are building OMPI and therefore have a full OMPI
@@ -375,12 +422,12 @@ typedef long long bool;
 #    define free(ptr) opal_free((ptr), __FILE__, __LINE__)
 
 /*
- * If we're mem debugging, make the OMPI_DEBUG_ZERO resolve to memset
+ * If we're mem debugging, make the OPAL_DEBUG_ZERO resolve to memset
  */
 #    include <string.h>
-#    define OMPI_DEBUG_ZERO(obj) memset(&(obj), 0, sizeof(obj))
+#    define OPAL_DEBUG_ZERO(obj) memset(&(obj), 0, sizeof(obj))
 #else
-#    define OMPI_DEBUG_ZERO(obj)
+#    define OPAL_DEBUG_ZERO(obj)
 #endif
 
 /*
@@ -415,7 +462,7 @@ typedef long long bool;
  * Some platforms (Solaris) have a broken qsort implementation.  Work
  * around by using our own.
  */
-#if OMPI_HAVE_BROKEN_QSORT
+#if OPAL_HAVE_BROKEN_QSORT
 #ifdef qsort
 #undef qsort
 #endif
@@ -502,6 +549,7 @@ static inline uint16_t ntohs(uint16_t netvar) { return netvar; }
 #define sin6_len sin_len
 #define sin6_family sin_family
 #define sin6_port sin_port
+#define sin6_addr sin_addr
 #endif
 
 #if !HAVE_DECL_AF_UNSPEC
