@@ -98,6 +98,12 @@ OPAL_DECLSPEC int opal_carto_base_select(void);
  */
 OPAL_DECLSPEC int opal_carto_base_close(void);
 
+/**
+ * Indication of whether a component was successfully selected or
+ * not
+ */
+OPAL_DECLSPEC extern bool opal_carto_base_selected;
+
 /**
  * Global component struct for the selected component
  */
@@ -190,6 +196,7 @@ OPAL_DECLSPEC extern opal_carto_graph_t *opal_carto_base_common_host_graph;
 
 OPAL_DECLSPEC extern opal_carto_base_module_1_0_0_t opal_carto_default_module;
 
+
 END_C_DECLS
 
 #endif /* OPAL_BASE_CARTO_H */
