@@ -21,12 +21,8 @@
 #define GRPCOMM_CNOS_H
 
 #include "orte_config.h"
-#include "orte/types.h"
 #include "orte/constants.h"
 
-#include "opal/threads/mutex.h"
-#include "opal/threads/condition.h"
-#include "opal/class/opal_object.h"
 
 #include "orte/mca/grpcomm/grpcomm.h"
 
