@@ -151,6 +151,21 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
         }
     }
     
+    /* now filter the list through any add-host specification */
+    if (NULL != app->add_host) {
+        if (ORTE_SUCCESS != (rc = orte_util_filter_dash_host_nodes(allocated_nodes,
+                                                                   app->add_host))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+        /** check that anything is left! */
+        if (0 == opal_list_get_size(allocated_nodes)) {
+            orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:no-mapped-node",
+                           true, app->app, "");
+            return ORTE_ERR_SILENT;
+        }
+    }
+    
     /* If the "no local" option was set, then remove the local node
      * from the list
      */
@@ -252,10 +267,11 @@ PROCESS:
      * in the mapper
      */
     OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base.rmaps_output,
-                         "%s rmaps:base: mapping proc for job %s to node %s",
+                         "%s rmaps:base: mapping proc for job %s to node %s whose daemon is %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          ORTE_JOBID_PRINT(proc->name.jobid),
-                         (NULL == node->name) ? "NULL" : node->name));
+                         (NULL == node->name) ? "NULL" : node->name,
+                         (NULL == node->daemon) ? "NULL" : ORTE_NAME_PRINT(&(node->daemon->name))));
     
     if (0 > (rc = opal_pointer_array_add(node->procs, (void*)proc))) {
         ORTE_ERROR_LOG(rc);
@@ -264,6 +280,9 @@ PROCESS:
     /* retain the proc struct so that we correctly track its release */
     OBJ_RETAIN(proc);
     ++node->num_procs;
+
+    /* update the oversubscribed state of the node */
+    node->oversubscribed = oversubscribed;
     
     return ORTE_SUCCESS;
 }
@@ -323,7 +342,7 @@ int orte_rmaps_base_claim_slot(orte_job_t *jdata,
                          ORTE_JOBID_PRINT(jdata->jobid), proc->app_idx, current_node->name));
     
     /* Be sure to demarcate the slots for this proc as claimed from the node */
-    current_node->slots_inuse += cpus_per_rank;
+    current_node->slots_inuse += 1;
     
     /* see if this node is oversubscribed now */
     if (current_node->slots_inuse > current_node->slots) {
@@ -371,7 +390,7 @@ int orte_rmaps_base_claim_slot(orte_job_t *jdata,
 int orte_rmaps_base_compute_vpids(orte_job_t *jdata)
 {
     orte_job_map_t *map;
-    orte_vpid_t vpid;
+    orte_vpid_t vpid, vpid_start;
     int i, j;
     orte_node_t *node;
     orte_proc_t *proc;
@@ -379,11 +398,40 @@ int orte_rmaps_base_compute_vpids(orte_job_t *jdata)
     
     map = jdata->map;
     
+    /* find the max vpid already assigned */
+    vpid_start = ORTE_VPID_MIN;
+    for (i=0; i < map->nodes->size; i++) {
+        if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
+            continue;
+        }
+        for (j=0; j < node->procs->size; j++) {
+            if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
+                continue;
+            }
+            /* ignore procs from other jobs */
+            if (proc->name.jobid != jdata->jobid) {
+                continue;
+            }
+            /* if the vpid is already defined, then update start */
+            if (ORTE_VPID_INVALID != proc->name.vpid &&
+                vpid_start < proc->name.vpid) {
+                vpid_start = proc->name.vpid;
+            }
+        }
+    }
+    if (ORTE_VPID_MIN == vpid_start) {
+        /* start at zero */
+        vpid_start = 0;
+    } else {
+        /* we start one higher than the max found */
+        vpid_start++;
+    }
+    
     if (ORTE_MAPPING_BYSLOT & map->policy ||
         ORTE_MAPPING_BYSOCKET & map->policy ||
         ORTE_MAPPING_BYBOARD & map->policy) {
         /* assign the ranks sequentially */
-        vpid = 0;
+        vpid = vpid_start;
         for (i=0; i < map->nodes->size; i++) {
             if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
                 continue;
@@ -397,21 +445,13 @@ int orte_rmaps_base_compute_vpids(orte_job_t *jdata)
                     continue;
                 }
                 if (ORTE_VPID_INVALID == proc->name.vpid) {
-                    /* find the next available vpid */
-                    for (vpid=0; vpid < jdata->num_procs; vpid++) {
-                        if (NULL == opal_pointer_array_get_item(jdata->procs, vpid)) {
-                            break;
-                        }
-                    }
-                    proc->name.vpid = vpid;
+                    proc->name.vpid = vpid++;
                 }
-                if (NULL == opal_pointer_array_get_item(jdata->procs, proc->name.vpid)) {
-                    if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs, proc->name.vpid, proc))) {
-                        ORTE_ERROR_LOG(rc);
-                        return rc;
-                    }                    
+                if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs,
+                                                                      proc->name.vpid, proc))) {
+                    ORTE_ERROR_LOG(rc);
+                    return rc;
                 }
-                vpid++;
             }
         }
         return ORTE_SUCCESS;
@@ -423,7 +463,7 @@ int orte_rmaps_base_compute_vpids(orte_job_t *jdata)
             if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
                 continue;
             }
-            vpid = i;
+            vpid = i + vpid_start;
             for (j=0; j < node->procs->size; j++) {
                 if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
                     continue;
@@ -433,21 +473,13 @@ int orte_rmaps_base_compute_vpids(orte_job_t *jdata)
                     continue;
                 }
                 if (ORTE_VPID_INVALID == proc->name.vpid) {
-                    /* find the next available vpid */
-                    vpid = i;
-                    while (NULL != opal_pointer_array_get_item(jdata->procs, vpid)) {
-                        vpid += map->num_nodes;
-                        if (jdata->num_procs <= vpid) {
-                            vpid = vpid - jdata->num_procs;
-                        }
-                    }
                     proc->name.vpid = vpid;
+                    vpid += map->num_nodes;
                 }
-                if (NULL == opal_pointer_array_get_item(jdata->procs, proc->name.vpid)) {
-                    if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs, proc->name.vpid, proc))) {
-                        ORTE_ERROR_LOG(rc);
-                        return rc;
-                    }                    
+                if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs,
+                                                                      proc->name.vpid, proc))) {
+                    ORTE_ERROR_LOG(rc);
+                    return rc;
                 }
             }
         }
