@@ -20,12 +20,17 @@
 #include "orte_config.h"
 #include "orte/constants.h"
 
-#include "orte/util/show_help.h"
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
+
 #include "opal/mca/base/base.h"
+#include "opal/util/output.h"
 #include "opal/mca/base/mca_base_param.h"
 #include "opal/mca/backtrace/backtrace.h"
 
 #include "orte/mca/rml/base/base.h"
+#include "orte/mca/rml/rml_types.h"
 #include "orte/mca/routed/routed.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/util/name_fns.h"
@@ -140,7 +145,7 @@ rml_oob_init(int* priority)
     /* Set default timeout for queued messages to be 1/2 second */
     orte_rml_oob_module.timeout.tv_sec = 0;
     orte_rml_oob_module.timeout.tv_usec = 500000;
-    orte_rml_oob_module.timer_event = malloc(sizeof(opal_event_t));
+    orte_rml_oob_module.timer_event = (opal_event_t *) malloc(sizeof(opal_event_t));
     if (NULL == orte_rml_oob_module.timer_event) {
         return NULL;
     }
@@ -500,7 +505,7 @@ rml_oob_recv_route_callback(int status,
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  ORTE_NAME_PRINT(&next)));
             ORTE_RML_OOB_MSG_HEADER_NTOH(*hdr);
-            qmsg->payload[0].iov_base = malloc(iov[0].iov_len);
+            qmsg->payload[0].iov_base = (IOVBASE_TYPE*) malloc(iov[0].iov_len);
             if (NULL == qmsg->payload[0].iov_base) abort();
             qmsg->payload[0].iov_len = iov[0].iov_len;
             memcpy(qmsg->payload[0].iov_base, iov[0].iov_base, iov[0].iov_len);
