@@ -11,18 +11,17 @@
 #include "orte_config.h"
 #include "orte/constants.h"
 
-#include "orte/util/show_help.h"
 #include "opal/threads/condition.h"
-#include "opal/runtime/opal_progress.h"
 #include "opal/dss/dss.h"
 #include "opal/class/opal_hash_table.h"
 #include "opal/class/opal_bitmap.h"
 #include "opal/util/bit_ops.h"
+#include "opal/util/output.h"
 
 #include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/grpcomm/grpcomm.h"
+#include "orte/mca/ess/ess.h"
 #include "orte/mca/rml/rml.h"
-#include "orte/mca/odls/odls_types.h"
+#include "orte/mca/rml/rml_types.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_wait.h"
@@ -43,11 +42,11 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat);
 static int route_lost(const orte_process_name_t *route);
 static bool route_is_defined(const orte_process_name_t *target);
 static int update_routing_tree(void);
-static orte_vpid_t get_routing_tree(orte_jobid_t job, opal_list_t *children);
-static bool proc_is_below(orte_vpid_t root, orte_vpid_t target);
+static orte_vpid_t get_routing_tree(opal_list_t *children);
 static int get_wireup_info(opal_buffer_t *buf);
+static int set_lifeline(orte_process_name_t *proc);
 
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
 static int binomial_ft_event(int state);
 #endif
 
@@ -60,11 +59,11 @@ orte_routed_module_t orte_routed_binomial_module = {
     init_routes,
     route_lost,
     route_is_defined,
+    set_lifeline,
     update_routing_tree,
     get_routing_tree,
-    proc_is_below,
     get_wireup_info,
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
     binomial_ft_event
 #else
     NULL
@@ -72,12 +71,11 @@ orte_routed_module_t orte_routed_binomial_module = {
 };
 
 /* local globals */
-static opal_hash_table_t        peer_list;
-static opal_hash_table_t        vpid_wildcard_list;
-static orte_process_name_t      wildcard_route;
+static opal_hash_table_t        jobfam_list;
 static opal_condition_t         cond;
 static opal_mutex_t             lock;
 static orte_process_name_t      *lifeline=NULL;
+static orte_process_name_t      local_lifeline;
 static orte_process_name_t      my_parent;
 static int                      num_children;
 static opal_list_t              my_children;
@@ -86,14 +84,8 @@ static bool                     ack_recvd;
 
 static int init(void)
 {
-    OBJ_CONSTRUCT(&peer_list, opal_hash_table_t);
-    opal_hash_table_init(&peer_list, 128);
-    
-    OBJ_CONSTRUCT(&vpid_wildcard_list, opal_hash_table_t);
-    opal_hash_table_init(&vpid_wildcard_list, 128);
-    
-    wildcard_route.jobid = ORTE_NAME_INVALID->jobid;
-    wildcard_route.vpid = ORTE_NAME_INVALID->vpid;
+    OBJ_CONSTRUCT(&jobfam_list, opal_hash_table_t);
+    opal_hash_table_init(&jobfam_list, 128);
     
     /* setup the global condition and lock */
     OBJ_CONSTRUCT(&cond, opal_condition_t);
@@ -112,16 +104,14 @@ static int init(void)
 static int finalize(void)
 {
     int rc;
-    uint64_t key;
-    void * value, *node, *next_node;
     opal_list_item_t *item;
     
     /* if I am an application process, indicate that I am
         * truly finalizing prior to departure
         */
-    if (!orte_process_info.hnp &&
-        !orte_process_info.daemon &&
-        !orte_process_info.tool) {
+    if (!ORTE_PROC_IS_HNP &&
+        !ORTE_PROC_IS_DAEMON &&
+        !ORTE_PROC_IS_TOOL) {
         if (ORTE_SUCCESS != (rc = orte_routed_base_register_sync(false))) {
             ORTE_ERROR_LOG(rc);
             return rc;
@@ -129,26 +119,11 @@ static int finalize(void)
     }
     
     /* if I am the HNP, I need to stop the comm recv */
-    if (orte_process_info.hnp) {
+    if (ORTE_PROC_IS_HNP) {
         orte_routed_base_comm_stop();
     }
     
-    /* don't destruct the routes until *after* we send the
-     * sync as the oob will be asking us how to route
-     * the message!
-     */
-    rc = opal_hash_table_get_first_key_uint64(&peer_list,
-                                              &key, &value, &node);
-    while(OPAL_SUCCESS == rc) {
-        if(NULL != value) {
-            free(value);
-        }
-        rc = opal_hash_table_get_next_key_uint64(&peer_list,
-                                                 &key, &value, node, &next_node);
-        node = next_node;
-    }
-    OBJ_DESTRUCT(&peer_list);
-    OBJ_DESTRUCT(&vpid_wildcard_list);
+    OBJ_DESTRUCT(&jobfam_list);
     /* destruct the global condition and lock */
     OBJ_DESTRUCT(&cond);
     OBJ_DESTRUCT(&lock);
@@ -178,8 +153,8 @@ static int delete_route(orte_process_name_t *proc)
     /* if I am an application process, I don't have any routes
      * so there is nothing for me to do
      */
-    if (!orte_process_info.hnp && !orte_process_info.daemon &&
-        !orte_process_info.tool) {
+    if (!ORTE_PROC_IS_HNP && !ORTE_PROC_IS_DAEMON &&
+        !ORTE_PROC_IS_TOOL) {
         return ORTE_SUCCESS;
     }
     
@@ -199,20 +174,20 @@ static int delete_route(orte_process_name_t *proc)
          * in my routing table and thus have nothing to do
          * here, just return
          */
-        if (orte_process_info.daemon) {
+        if (ORTE_PROC_IS_DAEMON) {
             return ORTE_SUCCESS;
         }
         
         /* see if this proc is present - it will have a wildcard vpid,
          * so we have to look for it with that condition
          */
-        rc = opal_hash_table_get_value_uint32(&vpid_wildcard_list,
+        rc = opal_hash_table_get_value_uint32(&jobfam_list,
                                               ORTE_JOB_FAMILY(proc->jobid),
                                               (void**)&route_copy);
         if (ORTE_SUCCESS == rc && NULL != route_copy) {
             /* proc is present - remove the data */
             free(route_copy);
-            rc = opal_hash_table_remove_value_uint32(&vpid_wildcard_list,
+            rc = opal_hash_table_remove_value_uint32(&jobfam_list,
                                                      ORTE_JOB_FAMILY(proc->jobid));
             if (ORTE_SUCCESS != rc) {
                 ORTE_ERROR_LOG(rc);
@@ -224,55 +199,12 @@ static int delete_route(orte_process_name_t *proc)
         return ORTE_SUCCESS;
     }
     
-    /* THIS CAME FROM OUR OWN JOB FAMILY... */
-    
-    /* treat vpid wildcards separately so they go onto the correct list */
-    if (proc->jobid != ORTE_JOBID_WILDCARD &&
-        proc->vpid == ORTE_VPID_WILDCARD) {
-        /* see if this target is already present - it will have a wildcard vpid,
-         * so we have to look for it on that list
-         */
-        rc = opal_hash_table_get_value_uint32(&vpid_wildcard_list,
-                                              proc->jobid,
-                                              (void**)&route_copy);
-        if (ORTE_SUCCESS == rc && NULL != route_copy) {
-            /* proc is present - remove the data */
-            free(route_copy);
-            rc = opal_hash_table_remove_value_uint32(&vpid_wildcard_list, proc->jobid);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
-            }            
-            return rc;
-        }
-        /* not already present - nothing to do */
-        return ORTE_SUCCESS;
-    }
-    
-    /* check for an exact match */
-    if (proc->jobid != ORTE_JOBID_WILDCARD &&
-        proc->vpid != ORTE_VPID_WILDCARD) {
-        /* see if this route already exists in our table */
-        rc = opal_hash_table_get_value_uint64(&peer_list,
-                                              orte_util_hash_name(proc),
-                                              (void**)&route_copy);
-        
-        if (ORTE_SUCCESS == rc && NULL != route_copy) {
-            /* proc is present - remove the data */
-            free(route_copy);
-            rc = opal_hash_table_remove_value_uint64(&peer_list, orte_util_hash_name(proc));
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
-            }            
-            return rc;
-        }
-        /* not already present - nothing to do */
-        return ORTE_SUCCESS;
-    }
-    
-    /* this must be a process that doesn't match any of the
-     * prior conditions - sorry!
+    /* THIS CAME FROM OUR OWN JOB FAMILY...there is nothing
+     * to do here. The routes will be redefined when we update
+     * the routing tree
      */
-    return ORTE_ERR_NOT_SUPPORTED;
+    
+    return ORTE_SUCCESS;
 }
 
 static int update_route(orte_process_name_t *target,
@@ -289,11 +221,17 @@ static int update_route(orte_process_name_t *target,
     /* if I am an application process, we don't update the route since
      * we automatically route everything through the local daemon
      */
-    if (!orte_process_info.hnp && !orte_process_info.daemon &&
-        !orte_process_info.tool) {
+    if (ORTE_PROC_IS_APP) {
         return ORTE_SUCCESS;
     }
 
+    /* if the job family is zero, then this is going to a local slave,
+     * so the path is direct and there is nothing to do here
+     */
+    if (0 == ORTE_JOB_FAMILY(target->jobid)) {
+        return ORTE_SUCCESS;
+    }
+    
     OPAL_OUTPUT_VERBOSE((1, orte_routed_base_output,
                          "%s routed_binomial_update: %s --> %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -310,7 +248,7 @@ static int update_route(orte_process_name_t *target,
          * anything to this job family via my HNP - so nothing to do
          * here, just return
          */
-        if (orte_process_info.daemon) {
+        if (ORTE_PROC_IS_DAEMON) {
             return ORTE_SUCCESS;
         }
         
@@ -323,7 +261,7 @@ static int update_route(orte_process_name_t *target,
         /* see if this target is already present - it will have a wildcard vpid,
          * so we have to look for it with that condition
          */
-        rc = opal_hash_table_get_value_uint32(&vpid_wildcard_list,
+        rc = opal_hash_table_get_value_uint32(&jobfam_list,
                                               ORTE_JOB_FAMILY(target->jobid),
                                               (void**)&route_copy);
         if (ORTE_SUCCESS == rc && NULL != route_copy) {
@@ -331,7 +269,7 @@ static int update_route(orte_process_name_t *target,
              * in case it has changed
              */
             *route_copy = *route;
-            rc = opal_hash_table_set_value_uint32(&vpid_wildcard_list,
+            rc = opal_hash_table_set_value_uint32(&jobfam_list,
                                                   ORTE_JOB_FAMILY(target->jobid), route_copy);
             if (ORTE_SUCCESS != rc) {
                 ORTE_ERROR_LOG(rc);
@@ -342,7 +280,7 @@ static int update_route(orte_process_name_t *target,
         /* not there, so add the route FOR THE JOB FAMILY*/
         route_copy = (orte_process_name_t *) malloc(sizeof(orte_process_name_t));
         *route_copy = *route;
-        rc = opal_hash_table_set_value_uint32(&vpid_wildcard_list,
+        rc = opal_hash_table_set_value_uint32(&jobfam_list,
                                               ORTE_JOB_FAMILY(target->jobid), route_copy);
         if (ORTE_SUCCESS != rc) {
             ORTE_ERROR_LOG(rc);
@@ -352,81 +290,17 @@ static int update_route(orte_process_name_t *target,
     
     /* THIS CAME FROM OUR OWN JOB FAMILY... */
     
-    /* treat vpid wildcards separately so they go onto the correct list */
-    if (target->jobid != ORTE_JOBID_WILDCARD &&
-        target->vpid == ORTE_VPID_WILDCARD) {
-        /* see if this target is already present - it will have a wildcard vpid,
-         * so we have to look for it on that list
-         */
-        rc = opal_hash_table_get_value_uint32(&vpid_wildcard_list,
-                                              target->jobid,
-                                              (void**)&route_copy);
-        if (ORTE_SUCCESS == rc && NULL != route_copy) {
-            /* target already present - update the route info
-             * in case it has changed
-             */
-            *route_copy = *route;
-            rc = opal_hash_table_set_value_uint32(&vpid_wildcard_list,
-                                                  target->jobid, route_copy);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
-            }            
-            return rc;
-        }
-        
-        /* not already present, so let's add it */
-        route_copy = (orte_process_name_t *) malloc(sizeof(orte_process_name_t));
-        *route_copy = *route;
-        rc = opal_hash_table_set_value_uint32(&vpid_wildcard_list,
-                                              target->jobid, route_copy);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
-        }
-        return rc;
-    }
+    opal_output(0, "%s CALL TO UPDATE ROUTE FOR OWN JOB FAMILY", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
     
-    /* check for an exact match */
-    if (target->jobid != ORTE_JOBID_WILDCARD &&
-        target->vpid != ORTE_VPID_WILDCARD) {
-        /* see if this route already exists in our table */
-        rc = opal_hash_table_get_value_uint64(&peer_list,
-                                              orte_util_hash_name(target),
-                                              (void**)&route_copy);
-        
-        if (ORTE_SUCCESS == rc && NULL != route_copy) {
-            /* target already present - update the route info
-             * in case it has changed
-             */
-            *route_copy = *route;
-            rc = opal_hash_table_set_value_uint64(&peer_list,
-                                                  orte_util_hash_name(target), route_copy);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
-            }            
-            return rc;
-        }
-        
-        /* not present - add it to the table */
-        route_copy = (orte_process_name_t *) malloc(sizeof(orte_process_name_t));
-        *route_copy = *route;
-        rc = opal_hash_table_set_value_uint64(&peer_list,
-                                              orte_util_hash_name(target), route_copy);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
-        }
-        return rc;
-    }
-
-    /* this must be a process that doesn't match any of the
-     * prior conditions - sorry!
-     */
     return ORTE_ERR_NOT_SUPPORTED;
 }
 
 
 static orte_process_name_t get_route(orte_process_name_t *target)
 {
-    orte_process_name_t *ret;
+    orte_process_name_t *ret, daemon;
+    opal_list_item_t *item;
+    orte_routed_tree_t *child;
     int rc;
 
     if (target->jobid == ORTE_JOBID_INVALID ||
@@ -442,16 +316,39 @@ static orte_process_name_t get_route(orte_process_name_t *target)
     }
     
     /* if I am an application process, always route via my local daemon */
-    if (!orte_process_info.hnp && !orte_process_info.daemon &&
-        !orte_process_info.tool) {
+    if (ORTE_PROC_IS_APP) {
         ret = ORTE_PROC_MY_DAEMON;
         goto found;
     }
 
+    /* if I am a tool, the route is direct if target is in
+     * my own job family, and to the target's HNP if not
+     */
+    if (ORTE_PROC_IS_TOOL) {
+        if (ORTE_JOB_FAMILY(target->jobid) == ORTE_JOB_FAMILY(ORTE_PROC_MY_NAME->jobid)) {
+            ret = target;
+            goto found;
+        } else {
+            ORTE_HNP_NAME_FROM_JOB(&daemon, target->jobid);
+            ret = &daemon;
+            goto found;
+        }
+    }
+    
+    /******     HNP AND DAEMONS ONLY     ******/
+    
+    /* if the job family is zero, then this is going to a local slave,
+     * so the path is direct
+     */
+    if (0 == ORTE_JOB_FAMILY(target->jobid)) {
+        ret = target;
+        goto found;
+    }
+    
     /* IF THIS IS FOR A DIFFERENT JOB FAMILY... */
     if (ORTE_JOB_FAMILY(target->jobid) != ORTE_JOB_FAMILY(ORTE_PROC_MY_NAME->jobid)) {
         /* if I am a daemon, route this via the HNP */
-        if (orte_process_info.daemon) {
+        if (ORTE_PROC_IS_DAEMON) {
             ret = ORTE_PROC_MY_HNP;
             goto found;
         }
@@ -459,7 +356,7 @@ static orte_process_name_t get_route(orte_process_name_t *target)
         /* if I am the HNP or a tool, then I stored a route to
          * this job family, so look it up
          */
-        rc = opal_hash_table_get_value_uint32(&vpid_wildcard_list,
+        rc = opal_hash_table_get_value_uint32(&jobfam_list,
                                               ORTE_JOB_FAMILY(target->jobid), (void**)&ret);
         if (ORTE_SUCCESS == rc) {
             /* got a good result - return it */
@@ -472,27 +369,58 @@ static orte_process_name_t get_route(orte_process_name_t *target)
      
     /* THIS CAME FROM OUR OWN JOB FAMILY... */
 
-    /* check exact matches */
-    rc = opal_hash_table_get_value_uint64(&peer_list,
-                                          orte_util_hash_name(target), (void**)&ret);
-    if (ORTE_SUCCESS == rc) {
-        /* got a good result - return it */
+    /* if we are not using static ports and this is going to the HNP, send direct */
+    if (!orte_static_ports &&
+        ORTE_PROC_MY_HNP->jobid == target->jobid &&
+        ORTE_PROC_MY_HNP->vpid == target->vpid) {
+        OPAL_OUTPUT_VERBOSE((2, orte_routed_base_output,
+                             "%s routing not enabled - going direct",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+        ret = target;
         goto found;
     }
     
-    /* didn't find an exact match - check to see if a route for this job was defined */
-    rc = opal_hash_table_get_value_uint32(&vpid_wildcard_list,
-                                          target->jobid, (void**)&ret);
-    if (ORTE_SUCCESS == rc) {
-        /* got a good result - return it */
+    daemon.jobid = ORTE_PROC_MY_NAME->jobid;
+    /* find out what daemon hosts this proc */
+    if (ORTE_VPID_INVALID == (daemon.vpid = orte_ess.proc_get_daemon(target))) {
+        ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+        ret = ORTE_NAME_INVALID;
         goto found;
     }
-    
-    /* default to wildcard route */
-    ret = &wildcard_route;
+  
+    /* if the daemon is me, then send direct to the target! */
+    if (ORTE_PROC_MY_NAME->vpid == daemon.vpid) {
+        ret = target;
+        goto found;
+    } else {
+        /* search routing tree for next step to that daemon */
+        for (item = opal_list_get_first(&my_children);
+             item != opal_list_get_end(&my_children);
+             item = opal_list_get_next(item)) {
+            child = (orte_routed_tree_t*)item;
+            if (child->vpid == daemon.vpid) {
+                /* the child is hosting the proc - just send it there */
+                ret = &daemon;
+                goto found;
+            }
+            /* otherwise, see if the daemon we need is below the child */
+            if (opal_bitmap_is_set_bit(&child->relatives, daemon.vpid)) {
+                /* yep - we need to step through this child */
+                daemon.vpid = child->vpid;
+                ret = &daemon;
+                goto found;
+            }
+        }
+    }
+
+    /* if we get here, then the target daemon is not beneath
+     * any of our children, so we have to step up through our parent
+     */
+    daemon.vpid = my_parent.vpid;
+    ret = &daemon;
 
  found:
-    OPAL_OUTPUT_VERBOSE((2, orte_routed_base_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_routed_base_output,
                          "%s routed_binomial_get(%s) --> %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          ORTE_NAME_PRINT(target), 
@@ -501,85 +429,6 @@ static orte_process_name_t get_route(orte_process_name_t *target)
     return *ret;
 }
 
-static int process_callback(orte_jobid_t job, opal_buffer_t *buffer)
-{
-    orte_proc_t **procs;
-    orte_job_t *jdata;
-    orte_std_cntr_t cnt;
-    char *rml_uri;
-    orte_process_name_t name;
-    int rc;
-    bool singleton=false;
-    
-    /* lookup the job object for this process */
-    if (NULL == (jdata = orte_get_job_data_object(job))) {
-        if (ORTE_JOB_FAMILY(job) == ORTE_JOB_FAMILY(ORTE_PROC_MY_NAME->jobid)) {
-            /* came from my job family - this is an error */
-            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-            return ORTE_ERR_NOT_FOUND;
-        }
-        /* must be from a singleton that is connecting to us - just add the contact info */
-        singleton = true;
-    } else {
-        procs = (orte_proc_t**)jdata->procs->addr;
-
-    }
-    
-    /* unpack the data for each entry */
-    cnt = 1;
-    while (ORTE_SUCCESS == (rc = opal_dss.unpack(buffer, &rml_uri, &cnt, OPAL_STRING))) {
-        
-        OPAL_OUTPUT_VERBOSE((2, orte_routed_base_output,
-                             "%s routed_binomial:callback got uri %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             (NULL == rml_uri) ? "NULL" : rml_uri));
-        
-        if (rml_uri == NULL) continue;
-        
-        /* we don't need to set the contact info into our rml
-         * hash table as we won't talk to the proc directly
-         */
-        
-        /* extract the proc's name */
-        if (ORTE_SUCCESS != (rc = orte_rml_base_parse_uris(rml_uri, &name, NULL))) {
-            ORTE_ERROR_LOG(rc);
-            free(rml_uri);
-            continue;
-        }
-        
-        if (!singleton) {
-            /* the procs are stored in vpid order, so update the record */
-            procs[name.vpid]->rml_uri = strdup(rml_uri);
-            
-            /* update the proc state */
-            if (procs[name.vpid]->state < ORTE_PROC_STATE_RUNNING) {
-                procs[name.vpid]->state = ORTE_PROC_STATE_RUNNING;
-            }
-            
-            ++jdata->num_reported;
-        }
-        free(rml_uri);
-
-        cnt = 1;
-    }
-    if (ORTE_ERR_UNPACK_READ_PAST_END_OF_BUFFER != rc) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
-    }    
-    
-    if (!singleton) {
-        /* if all procs have reported, update our job state */
-        if (jdata->num_reported == jdata->num_procs) {
-            /* update the job state */
-            if (jdata->state < ORTE_JOB_STATE_RUNNING) {
-                jdata->state = ORTE_JOB_STATE_RUNNING;
-            }
-        }
-    }
-    
-    return ORTE_SUCCESS;
-}
-
 /* HANDLE ACK MESSAGES FROM AN HNP */
 static void release_ack(int fd, short event, void *data)
 {
@@ -619,7 +468,7 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
     int rc;
 
     /* if I am a tool, then I stand alone - there is nothing to do */
-    if (orte_process_info.tool) {
+    if (ORTE_PROC_IS_TOOL) {
         return ORTE_SUCCESS;
     }
     
@@ -627,7 +476,7 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
      * from the data sent to me for launch and update the routing tables to
      * point at the daemon for each proc
      */
-    if (orte_process_info.daemon) {
+    if (ORTE_PROC_IS_DAEMON) {
         
         OPAL_OUTPUT_VERBOSE((1, orte_routed_base_output,
                              "%s routed_binomial: init routes for daemon job %s\n\thnp_uri %s",
@@ -657,21 +506,13 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
                 return rc;
             }
 
-            /* if ndat is NULL, then this is being called during init,
-             * so just seed the routing table with a path back to the HNP...
-             */
-            if (ORTE_SUCCESS != (rc = update_route(ORTE_PROC_MY_HNP, ORTE_PROC_MY_HNP))) {
-                ORTE_ERROR_LOG(rc);
-                return rc;
+            /* if we are using static ports, set my lifeline to point at my parent */
+            if (orte_static_ports) {
+                lifeline = &my_parent;
+            } else {
+                /* set our lifeline to the HNP - we will abort if that connection is lost */
+                lifeline = ORTE_PROC_MY_HNP;
             }
-            /* set the wildcard route for anybody whose name we don't recognize
-             * to be the HNP
-             */
-            wildcard_route.jobid = ORTE_PROC_MY_HNP->jobid;
-            wildcard_route.vpid = ORTE_PROC_MY_HNP->vpid;
-            
-            /* set our lifeline to the the HNP - we will abort if that connection is lost */
-            lifeline = ORTE_PROC_MY_HNP;
             
             /* daemons will send their contact info back to the HNP as
              * part of the message confirming they are read to go. HNP's
@@ -695,7 +536,7 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
     }
     
 
-    if (orte_process_info.hnp) {
+    if (ORTE_PROC_IS_HNP) {
         
         OPAL_OUTPUT_VERBOSE((1, orte_routed_base_output,
                              "%s routed_binomial: init routes for HNP job %s",
@@ -723,7 +564,7 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
                 }
             } else {
                 /* if not, then I need to process the callback */
-                if (ORTE_SUCCESS != (rc = process_callback(job, ndat))) {
+                if (ORTE_SUCCESS != (rc = orte_routed_base_process_callback(job, ndat))) {
                     ORTE_ERROR_LOG(rc);
                     return rc;
                 }
@@ -751,17 +592,32 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
                                  "%s routed_binomial: init routes w/non-NULL data",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
             
-            /* if this is for my job family, then we send the buffer
-             * to the proper tag on the daemon
+            /* if this is for a job family of zero, then we know that the enclosed
+             * procs are local slaves to our daemon. In that case, we can just ignore this
+             * as our daemon - given that it had to spawn the local slave - already
+             * knows how to talk to them
              */
-            if (ORTE_JOB_FAMILY(ORTE_PROC_MY_NAME->jobid) == ORTE_JOB_FAMILY(job)) {
-                /* send the buffer to the proper tag on the daemon */
-                if (0 > (rc = orte_rml.send_buffer(ORTE_PROC_MY_DAEMON, ndat,
+            if (0 == ORTE_JOB_FAMILY(job)) {
+                return ORTE_SUCCESS;
+            }
+            
+            if (ORTE_JOB_FAMILY(ORTE_PROC_MY_NAME->jobid) != ORTE_JOB_FAMILY(job)) {
+                /* if this is for a different job family, then we route via our HNP
+                 * to minimize connection counts to entities such as ompi-server, so
+                 * start by sending the contact info to the HNP for update
+                 */
+                OPAL_OUTPUT_VERBOSE((1, orte_routed_base_output,
+                                     "%s routed_binomial_init_routes: diff job family - sending update to %s",
+                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                     ORTE_NAME_PRINT(ORTE_PROC_MY_HNP)));
+                
+                if (0 > (rc = orte_rml.send_buffer(ORTE_PROC_MY_HNP, ndat,
                                                    ORTE_RML_TAG_RML_INFO_UPDATE, 0))) {
                     ORTE_ERROR_LOG(rc);
                     return rc;
                 }
-                /* wait right here until the daemon acks the update to ensure that
+                
+                /* wait right here until the HNP acks the update to ensure that
                  * any subsequent messaging can succeed
                  */
                 ack_recvd = false;
@@ -774,43 +630,10 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
                                      "%s routed_binomial_init_routes: ack recvd",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
                 
-                /* we already have defined our routes to everyone to
-                 * be through the local daemon, so nothing further to do
+                /* our get_route function automatically routes all messages for
+                 * other job families via the HNP, so nothing more to do here
                  */
-                return ORTE_SUCCESS;
             }
-            
-            /* if this is for a different job family, then we route via our HNP
-             * to minimize connection counts to entities such as ompi-server, so
-             * start by sending the contact info to the HNP for update
-             */
-            OPAL_OUTPUT_VERBOSE((1, orte_routed_base_output,
-                                 "%s routed_binomial_init_routes: diff job family - sending update to %s",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_HNP)));
-            
-            if (0 > (rc = orte_rml.send_buffer(ORTE_PROC_MY_HNP, ndat,
-                                               ORTE_RML_TAG_RML_INFO_UPDATE, 0))) {
-                ORTE_ERROR_LOG(rc);
-                return rc;
-            }
-            
-            /* wait right here until the HNP acks the update to ensure that
-             * any subsequent messaging can succeed
-             */
-            ack_recvd = false;
-            rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_UPDATE_ROUTE_ACK,
-                                         ORTE_RML_NON_PERSISTENT, recv_ack, NULL);
-            
-            ORTE_PROGRESSED_WAIT(ack_recvd, 0, 1);
-            
-            OPAL_OUTPUT_VERBOSE((1, orte_routed_base_output,
-                                 "%s routed_binomial_init_routes: ack recvd",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-            
-            /* our get_route function automatically routes all messages for
-             * other job families via the HNP, so nothing more to do here
-             */
             return ORTE_SUCCESS;
         }
         
@@ -864,10 +687,6 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
             return rc;
         }
         
-        /* setup the route to all other procs to flow through the daemon */
-        wildcard_route.jobid = ORTE_PROC_MY_DAEMON->jobid;
-        wildcard_route.vpid = ORTE_PROC_MY_DAEMON->vpid;
-        
         /* set our lifeline to the local daemon - we will abort if this connection is lost */
         lifeline = ORTE_PROC_MY_DAEMON;
         
@@ -886,7 +705,6 @@ static int init_routes(orte_jobid_t job, opal_buffer_t *ndat)
          *     is attempted until the overall ORTE system knows how to talk to everyone -
          *     otherwise, the system can just hang.
          */
-        
         if (ORTE_SUCCESS != (rc = orte_routed_base_register_sync(true))) {
             ORTE_ERROR_LOG(rc);
             return rc;
@@ -918,32 +736,27 @@ static int route_lost(const orte_process_name_t *route)
 }
 
 
-
-/******* stub functions - to be implemented ******/
 static bool route_is_defined(const orte_process_name_t *target)
 {
+    /* find out what daemon hosts this proc */
+    if (ORTE_VPID_INVALID == orte_ess.proc_get_daemon((orte_process_name_t*)target)) {
+        return false;
+    }
+    
     return true;
 }
 
-/*************************************/
-typedef struct {
-    opal_list_item_t super;
-    orte_vpid_t vpid;
-    opal_bitmap_t relatives;
-} orte_routed_tree_t;
-
-static void construct(orte_routed_tree_t *rt)
+static int set_lifeline(orte_process_name_t *proc)
 {
-    rt->vpid = ORTE_VPID_INVALID;
-    OBJ_CONSTRUCT(&rt->relatives, opal_bitmap_t);
+    /* we have to copy the proc data because there is no
+     * guarantee that it will be preserved
+     */
+    local_lifeline.jobid = proc->jobid;
+    local_lifeline.vpid = proc->vpid;
+    lifeline = &local_lifeline;
+    
+    return ORTE_SUCCESS;
 }
-static void destruct(orte_routed_tree_t *rt)
-{
-    OBJ_DESTRUCT(&rt->relatives);
-}
-OBJ_CLASS_INSTANCE(orte_routed_tree_t, opal_list_item_t,
-                   construct, destruct);
-
 
 static int binomial_tree(int rank, int parent, int me, int num_procs,
                          int *nchildren, opal_list_t *childrn, opal_bitmap_t *relatives)
@@ -1016,7 +829,7 @@ static int update_routing_tree(void)
     /* if I am anything other than a daemon or the HNP, this
      * is a meaningless command as I am not allowed to route
      */
-    if (!orte_process_info.daemon && !orte_process_info.hnp) {
+    if (!ORTE_PROC_IS_DAEMON && !ORTE_PROC_IS_HNP) {
         return ORTE_ERR_NOT_SUPPORTED;
     }
     
@@ -1051,16 +864,15 @@ static int update_routing_tree(void)
     return ORTE_SUCCESS;
 }
 
-static orte_vpid_t get_routing_tree(orte_jobid_t job, opal_list_t *children)
+static orte_vpid_t get_routing_tree(opal_list_t *children)
 {
     opal_list_item_t *item;
-    orte_namelist_t *nm;
-    orte_routed_tree_t *child;
+    orte_routed_tree_t *child, *nm;
     
     /* if I am anything other than a daemon or the HNP, this
      * is a meaningless command as I am not allowed to route
      */
-    if (!orte_process_info.daemon && !orte_process_info.hnp) {
+    if (!ORTE_PROC_IS_DAEMON && !ORTE_PROC_IS_HNP) {
         return ORTE_VPID_INVALID;
     }
     
@@ -1072,12 +884,13 @@ static orte_vpid_t get_routing_tree(orte_jobid_t job, opal_list_t *children)
              item != opal_list_get_end(&my_children);
              item = opal_list_get_next(item)) {
             child = (orte_routed_tree_t*)item;
-            nm = OBJ_NEW(orte_namelist_t);
-            nm->name.jobid = ORTE_PROC_MY_NAME->jobid;
-            nm->name.vpid = child->vpid;
-            opal_list_append(children, &nm->item);
+            nm = OBJ_NEW(orte_routed_tree_t);
+            nm->vpid = child->vpid;
+            opal_bitmap_copy(&nm->relatives, &child->relatives);
+            opal_list_append(children, &nm->super);
         }
     }
+    
     /* return my parent's vpid */
     return my_parent.vpid;
 }
@@ -1090,7 +903,7 @@ static int get_wireup_info(opal_buffer_t *buf)
      * is a meaningless command as I cannot get
      * the requested info
      */
-    if (!orte_process_info.hnp) {
+    if (!ORTE_PROC_IS_HNP) {
         return ORTE_ERR_NOT_SUPPORTED;
     }
     
@@ -1110,44 +923,8 @@ static int get_wireup_info(opal_buffer_t *buf)
     return ORTE_SUCCESS;
 }
 
-static bool proc_is_below(orte_vpid_t root, orte_vpid_t target)
-{
-    opal_list_item_t *item;
-    orte_routed_tree_t *child;
-    
-    /* if I am anything other than a daemon or the HNP, this
-     * is a meaningless command as I am not allowed to route
-     */
-    if (!orte_process_info.daemon && !orte_process_info.hnp) {
-        return false;
-    }
 
-    /* quick check: if root == target, then the answer is always true! */
-    if (root == target) {
-        return true;
-    }
-    
-    /* check the list of children to see if either their vpid
-     * matches target, or the target bit is set in their bitmap
-     */
-    
-    /* first find the specified child */
-    for (item = opal_list_get_first(&my_children);
-         item != opal_list_get_end(&my_children);
-         item = opal_list_get_next(item)) {
-        child = (orte_routed_tree_t*)item;
-        if (child->vpid == root) {
-            /* now see if the target lies below this child */
-            return opal_bitmap_is_set_bit(&child->relatives, target);
-        }
-    }
-    
-    /* only get here if we have no children or we didn't find anything */
-    return false;
-}
-
-
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
 static int binomial_ft_event(int state)
 {
     int ret, exit_status = ORTE_SUCCESS;
